@@ -1,11 +1,11 @@
-// Copyright (c) 2019 ETH Zurich, Mattia Gollub (mattia.gollub@bsse.ethz.ch)
+// Copyright (c) 2021 ETH Zurich, Mattia Gollub (mattia.gollub@bsse.ethz.ch)
 // Computational Systems Biology group, D-BSSE
 //
 // This software is freely available under the GNU General Public License v3.
 // See the LICENSE file or http://www.gnu.org/licenses/ for further information.
 
-#ifndef HYPERFLUX_STEADY_STATE_VERIFIER_H
-#define HYPERFLUX_STEADY_STATE_VERIFIER_H
+#ifndef PTA_STEADY_STATE_VERIFIER_H
+#define PTA_STEADY_STATE_VERIFIER_H
 
 #include <gurobi_c++.h>
 
@@ -14,20 +14,21 @@
 #include <set>
 #include <vector>
 
+#include "Eigen/src/Core/util/Meta.h"
+#include "commons.h"
 #include "eigen-hash.h"
 #include "flux_constraints.h"
 #include "lrucache.hpp"
 #include "rays_packet.h"
 
-namespace hyperflux {
+namespace pta {
 
 /**
  * @brief Class for the verification of the steady state of the steady state
  * condition in a reaction network.
  */
-template <typename Scalar>
-class SteadyStateVerifier {
-public:
+template <typename Scalar> class SteadyStateVerifier {
+  public:
     /**
      * @brief Construct a new instance of the SteadyStateVerifier class.
      *
@@ -57,13 +58,13 @@ public:
         const samply::Matrix<Scalar>& v_maxs,
         const Eigen::Matrix<Eigen::Index, -1, 1>& constrained_rxn_ids);
 
-    std::vector<size_t> get_steady_state_regions_ids(
-        const samply::RaysPacket<Scalar>& drg_rays,
-        const samply::Vector<Scalar>& region_starts,
-        const samply::Vector<Scalar>& region_ends,
-        const Eigen::VectorXi& ray_indices);
+    std::vector<size_t>
+    get_steady_state_regions_ids(const samply::RaysPacket<Scalar>& drg_rays,
+                                 const samply::Vector<Scalar>& region_starts,
+                                 const samply::Vector<Scalar>& region_ends,
+                                 const samply::IndexVector& ray_indices);
 
-private:
+  private:
     typedef cache::lru_cache<samply::Vector<Scalar>, bool> feasibility_cache;
     /**
      * @brief Create variables representing the reaction fluxes. The objective
@@ -73,8 +74,8 @@ private:
      * @param num_reactions The number of reactions in the network.
      * @return Array containing the created variables.
      */
-    std::unique_ptr<GRBVar[]> create_v_variables(
-        const FluxConstraints& flux_constraints);
+    std::unique_ptr<GRBVar[]>
+    create_v_variables(const FluxConstraints& flux_constraints);
 
     /**
      * @brief Create the linear flux constraints based on the stoichiometry. The
@@ -85,7 +86,7 @@ private:
      */
     std::unique_ptr<GRBConstr[]> create_v_constraints(const samply::Matrix<double>& S);
 
-    void create_stedy_state_conditions_(const FluxConstraints& flux_constraints);
+    void create_steady_state_conditions_(const FluxConstraints& flux_constraints);
 
     bool is_origin_region_(const Scalar region_start, const Scalar region_end);
 
@@ -95,7 +96,7 @@ private:
 
     std::vector<Eigen::Index> find_constrained_rev_reaction_ids_(
         const Eigen::Matrix<Eigen::Index, -1, 1> has_drg_reaction_ids,
-        const FluxConstraints& flux_constriants);
+        const FluxConstraints& flux_constraints);
 
     // The gurobi environment in which the model is created.
     const GRBEnv gurobi_environment;
@@ -138,16 +139,13 @@ SteadyStateVerifier<Scalar>::SteadyStateVerifier(
     const Eigen::Matrix<Eigen::Index, -1, 1>& has_drg_reaction_ids,
     const double flux_threshold,
     const size_t cache_size)
-    : gurobi_environment(),
-      model(gurobi_environment),
+    : gurobi_environment(), model(gurobi_environment),
       v_variables(create_v_variables(flux_constraints)),
       v_constraints(create_v_constraints(flux_constraints.S)),
-      flux_constraints_(flux_constraints),
-      has_drg_reaction_ids_(has_drg_reaction_ids),
+      flux_constraints_(flux_constraints), has_drg_reaction_ids_(has_drg_reaction_ids),
       has_drg_rev_reaction_ids_(
           find_constrained_rev_reaction_ids_(has_drg_reaction_ids, flux_constraints)),
-      flux_threshold_(static_cast<Scalar>(flux_threshold)),
-      cache_(cache_size)
+      flux_threshold_(static_cast<Scalar>(flux_threshold)), cache_(cache_size)
 {
     // Use one thread only, since the sampling process is already parallelized
     // at the samples level.
@@ -155,7 +153,7 @@ SteadyStateVerifier<Scalar>::SteadyStateVerifier(
     model.set(GRB_IntParam_OutputFlag, 0);
     model.set(GRB_DoubleParam_FeasibilityTol, 1e-9);
 
-    create_stedy_state_conditions_(flux_constraints);
+    create_steady_state_conditions_(flux_constraints);
 }
 
 template <typename Scalar>
@@ -163,7 +161,7 @@ std::vector<size_t> SteadyStateVerifier<Scalar>::get_steady_state_regions_ids(
     const samply::RaysPacket<Scalar>& rev_drg_rays,
     const samply::Vector<Scalar>& region_starts,
     const samply::Vector<Scalar>& region_ends,
-    const Eigen::VectorXi& ray_indices)
+    const samply::IndexVector& ray_indices)
 {
     std::vector<size_t> steady_state_region_ids;
     steady_state_region_ids.reserve(region_starts.size());
@@ -171,11 +169,12 @@ std::vector<size_t> SteadyStateVerifier<Scalar>::get_steady_state_regions_ids(
     for (Eigen::Index region_idx = 0; region_idx < region_starts.size(); ++region_idx) {
         const Scalar region_start = region_starts(region_idx);
         const Scalar region_end = region_ends(region_idx);
-        const int ray_index = ray_indices(region_idx);
+        samply::Index ray_index = ray_indices(region_idx);
 
         if (is_origin_region_(region_start, region_end)) {
             steady_state_region_ids.push_back(region_idx);
-        } else {
+        }
+        else {
             const auto region_mid_t = (region_end + region_start) / 2.0;
             const auto region_mid_drg = rev_drg_rays.at(ray_index, region_mid_t);
             if (is_steady_state_region_(region_mid_drg))
@@ -213,9 +212,9 @@ inline bool SteadyStateVerifier<Scalar>::is_steady_state_region_(
 
     const samply::Vector<Scalar> v_mins =
         (rev_drg.array() < Scalar(0))
-            .select(
-                samply::Vector<Scalar>::Ones(num_constraints) * flux_threshold_ * 0.001,
-                -infinity);
+            .select(samply::Vector<Scalar>::Ones(num_constraints) * flux_threshold_ *
+                        0.001,
+                    -infinity);
     const samply::Vector<Scalar> v_maxs =
         (rev_drg.array() > Scalar(0))
             .select(samply::Vector<Scalar>::Ones(num_constraints) * -flux_threshold_ *
@@ -252,8 +251,8 @@ inline bool SteadyStateVerifier<Scalar>::is_influx_outflux_consistent_(
 }
 
 template <typename Scalar>
-inline std::unique_ptr<GRBVar[]> SteadyStateVerifier<Scalar>::create_v_variables(
-    const FluxConstraints& flux_constraints)
+inline std::unique_ptr<GRBVar[]>
+SteadyStateVerifier<Scalar>::create_v_variables(const FluxConstraints& flux_constraints)
 {
     return std::unique_ptr<GRBVar[]>(
         model.addVars(flux_constraints.lower_bounds.data(),
@@ -262,8 +261,8 @@ inline std::unique_ptr<GRBVar[]> SteadyStateVerifier<Scalar>::create_v_variables
 }
 
 template <typename Scalar>
-inline std::unique_ptr<GRBConstr[]> SteadyStateVerifier<Scalar>::create_v_constraints(
-    const Eigen::MatrixXd& S)
+inline std::unique_ptr<GRBConstr[]>
+SteadyStateVerifier<Scalar>::create_v_constraints(const Eigen::MatrixXd& S)
 {
     const size_t num_metabolites = S.rows();
     const size_t num_reactions = S.cols();
@@ -298,7 +297,7 @@ inline std::unique_ptr<GRBConstr[]> SteadyStateVerifier<Scalar>::create_v_constr
 }
 
 template <typename Scalar>
-inline void SteadyStateVerifier<Scalar>::create_stedy_state_conditions_(
+inline void SteadyStateVerifier<Scalar>::create_steady_state_conditions_(
     const FluxConstraints& flux_constraints)
 {
     const size_t num_metabolites = flux_constraints.S.rows();
@@ -346,7 +345,7 @@ template <typename ScalarType>
 inline std::vector<Eigen::Index>
 SteadyStateVerifier<ScalarType>::find_constrained_rev_reaction_ids_(
     const Eigen::Matrix<Eigen::Index, -1, 1> has_drg_reaction_ids,
-    const FluxConstraints& flux_constriants)
+    const FluxConstraints& flux_constraints)
 {
     std::vector<Eigen::Index> constrained_rev_reactions_ids;
     constrained_rev_reactions_ids.reserve(has_drg_reaction_ids.size());
@@ -362,6 +361,6 @@ SteadyStateVerifier<ScalarType>::find_constrained_rev_reaction_ids_(
     return constrained_rev_reactions_ids;
 }
 
-}  // namespace hyperflux
+} // namespace pta
 
 #endif
\ No newline at end of file
