@@ -11,21 +11,20 @@
 
 namespace samply {
 
-template <typename Scalar>
-struct AxisRaysPacket {
+template <typename Scalar> struct AxisRaysPacket {
     Matrix<Scalar> origins;
-    Vector<Eigen::Index> axis_ids;
+    IndexVector axis_ids;
 
     Matrix<Scalar> at(const Vector<Scalar>& t) const
     {
         Matrix<Scalar> result = origins;
-        for (size_t i = 0u; i < t.size(); ++i)
+        for (Eigen::Index i = 0u; i < t.size(); ++i)
             result(axis_ids(i), i) += t(i);
 
         return result;
     }
 
-    Matrix<Scalar> at(const Eigen::VectorXi& ray_indices, const Vector<Scalar>& t) const
+    Matrix<Scalar> at(const IndexVector& ray_indices, const Vector<Scalar>& t) const
     {
         Matrix<Scalar> result = origins(Eigen::all, ray_indices);
         for (const auto& i : ray_indices)
@@ -34,7 +33,7 @@ struct AxisRaysPacket {
         return result;
     }
 
-    Vector<Scalar> at(const int& ray_index, const Scalar t) const
+    Vector<Scalar> at(Index& ray_index, const Scalar t) const
     {
         Vector<Scalar> result = origins(Eigen::all, ray_index);
         result(axis_ids(ray_index), ray_index) += t(ray_index);
@@ -43,6 +42,6 @@ struct AxisRaysPacket {
     }
 };
 
-}  // namespace samply
+} // namespace samply
 
 #endif
\ No newline at end of file
