@@ -113,7 +113,7 @@ inline Vector<ScalarType> MixedMvnUniformPdfSampler<ScalarType>::sample1d(
     const IntersectionsPacket<Scalar>& in_intersections,
     SamplingHelper& sampling_helper)
 {
-    int n_rays = in_rays.origins.cols();
+    Eigen::Index n_rays = in_rays.origins.cols();
     std::vector<Eigen::Index> non_degenerate_ray_ids;
     std::vector<Eigen::Index> degenerate_ray_ids;
     non_degenerate_ray_ids.reserve(n_rays);
@@ -127,7 +127,12 @@ inline Vector<ScalarType> MixedMvnUniformPdfSampler<ScalarType>::sample1d(
     }
 
     Vector<ScalarType> t(n_rays);
-    if (non_degenerate_ray_ids.size() > 0)
+    if (degenerate_ray_ids.size() == 0)
+        // Special case qhere all the rays are non-degenerate.
+        t = MvnPdfSampler<Scalar>::sample1d(
+            in_rays, in_intersections,
+            sampling_helper);
+    else if (non_degenerate_ray_ids.size() > 0)
         // For the non-degenerated rays (i.e. rays that have support from the
         // MVN distributed parameters), sample the next points according to the
         // MVN distribution.
@@ -156,8 +161,9 @@ template <typename ScalarType>
 ReparametrizedObject<MixedMvnUniformPdfSampler<ScalarType>>
 MixedMvnUniformPdfSampler<ScalarType>::get_optimally_reparametrized_sampler() const
 {
+    // TODO: computation of the inverse fails.
     AffineTransform<double> from_optimal_parametrization =
-        vars_to_mvn_vars_.inverse() * MvnPdfSampler<ScalarType>::samples_transform_;
+       vars_to_mvn_vars_.inverse() * MvnPdfSampler<ScalarType>::samples_transform_;
     AffineTransform<double> to_optimal_parametrization =
         MvnPdfSampler<ScalarType>::samples_transform_.inverse() * vars_to_mvn_vars_;
 
