@@ -8,6 +8,7 @@
 #define SAMPLY_ELLIPSOID_H
 
 #include <Eigen/Dense>
+#include <cmath>
 #include <exception>
 #include <utility>
 
@@ -46,6 +47,14 @@ public:
      */
     Ellipsoid(const Eigen::Index n_dimensions);
 
+    /**
+     * @brief Construct a new Ellipsoid object representing the a hypersphere with the
+     * given radius.
+     *
+     * @param n_dimensions Number of dimensions of the ellipsoid.
+     */
+    Ellipsoid(const Eigen::Index n_dimensions, const double radius);
+
     /**
      * @brief Find the intersections between a set of rays and the ellipsoid.
      *
@@ -54,7 +63,9 @@ public:
      * @return An object describing all the intersections of the rays with the
      * ellipsoid.
      */
-    IntersectionsPacket<Scalar> intersect(const RaysPacket<Scalar>& rays) const;
+    template <typename RaysPacketType>
+    IntersectionsPacket<Scalar> intersect(
+        const RaysPacketBase<ScalarType, RaysPacketType>& rays) const;
 
     /**
      * @brief Get a reparametrized version of this object such that intersection
@@ -63,8 +74,8 @@ public:
      * @return Reparametrized version of this object, containing the
      * transforms mapping between the two parametrizations.
      */
-    ReparametrizedObject<Ellipsoid<Scalar>> get_optimally_reparametrized_descriptor(
-        const std::string& optimality_criterion) const;
+    ReparametrizedObject<Ellipsoid<Scalar>> get_optimally_reparametrized_descriptor()
+        const;
 
     /**
      * @brief Get the dimensionality of the ellipsoid.
@@ -92,8 +103,11 @@ private:
 
     const Matrix<Scalar> E_inv_;
     const Vector<Scalar> f_;
+    const double scale_;
+    const double scale_squared_;
 
     const bool is_standard_;
+    const bool is_scaled_standard_;
 };
 
 //==============================================================================
@@ -108,7 +122,10 @@ Ellipsoid<ScalarType>::Ellipsoid(const Matrix<double>& E, const Vector<double>&
               E.completeOrthogonalDecomposition().pseudoInverse())
                  .cast<ScalarType>()),
       f_(f.cast<ScalarType>()),
-      is_standard_(E.isIdentity() && f.isZero())
+      is_standard_(E.isIdentity() && f.isZero()),
+      is_scaled_standard_(false),
+      scale_(std::nan("")),
+      scale_squared_(std::nan(""))
 {
 }
 
@@ -118,13 +135,30 @@ Ellipsoid<ScalarType>::Ellipsoid(const Eigen::Index n_dimensions)
       f_full_precision(Vector<double>::Zero(n_dimensions)),
       E_inv_(Matrix<ScalarType>::Identity(n_dimensions, n_dimensions)),
       f_(Vector<ScalarType>::Zero(n_dimensions)),
-      is_standard_(true)
+      is_standard_(true),
+      is_scaled_standard_(false),
+      scale_(1.0),
+      scale_squared_(1.0)
 {
 }
 
 template <typename ScalarType>
+Ellipsoid<ScalarType>::Ellipsoid(const Eigen::Index n_dimensions, const double scale)
+    : E_full_precision(Matrix<double>::Identity(n_dimensions, n_dimensions)),
+      f_full_precision(Vector<double>::Zero(n_dimensions)),
+      E_inv_(Matrix<ScalarType>::Identity(n_dimensions, n_dimensions)),
+      f_(Vector<ScalarType>::Zero(n_dimensions)),
+      is_standard_(false),
+      is_scaled_standard_(true),
+      scale_(scale),
+      scale_squared_(scale * scale)
+{
+}
+
+template <typename ScalarType>
+template <typename RaysPacketType>
 IntersectionsPacket<ScalarType> Ellipsoid<ScalarType>::intersect(
-    const RaysPacket<ScalarType>& rays) const
+    const RaysPacketBase<ScalarType, RaysPacketType>& rays) const
 {
     // The intersection can be determined by solving the equation :
     // t ^ 2 * d*inv(E)*d + t * 2 * d*inv(E)*(o - f) + (o - f)*inv(E)*(o - f) - 1 = 0
@@ -139,6 +173,12 @@ IntersectionsPacket<ScalarType> Ellipsoid<ScalarType>::intersect(
         a = rays.directions.colwise().squaredNorm();
         b = 2 * rays.directions.cwiseProduct(rays.origins).colwise().sum();
         c = rays.origins.colwise().squaredNorm().array() - 1;
+    } else if (is_scaled_standard_) {
+        // If this is a scaled standard ellipsoid we can assume that
+        // E = I * scale and f = 0.
+        a = rays.directions.colwise().squaredNorm();
+        b = 2 * rays.directions.cwiseProduct(rays.origins).colwise().sum();
+        c = rays.origins.colwise().squaredNorm().array() - scale_squared_;
     } else {
         // Otherwise use the formula for the general case.
         auto offsets = (rays.origins.colwise() - f_);
@@ -163,23 +203,13 @@ IntersectionsPacket<ScalarType> Ellipsoid<ScalarType>::intersect(
 
 template <typename ScalarType>
 ReparametrizedObject<Ellipsoid<ScalarType>>
-Ellipsoid<ScalarType>::get_optimally_reparametrized_descriptor(
-    const std::string& optimality_criterion) const
+Ellipsoid<ScalarType>::get_optimally_reparametrized_descriptor() const
 {
-    if (optimality_criterion.compare("isotropy") == 0 ||
-        optimality_criterion.compare("intersection") == 0) {
-        AffineTransform<double> from_parametrization =
-            AffineTransform<double>(E_full_precision, f_full_precision);
-        return ReparametrizedObject<Ellipsoid<Scalar>>(
-            Ellipsoid<Scalar>(E_full_precision.cols()), from_parametrization,
-            from_parametrization.inverse());
-    } else {
-        throw std::runtime_error(
-            std::string("The '") + optimality_criterion +
-            std::string("' "
-                        "optimality criterion is not recognized by the Ellipsoid "
-                        "descriptor"));
-    }
+    AffineTransform<double> from_parametrization =
+        AffineTransform<double>(E_full_precision, f_full_precision);
+    return ReparametrizedObject<Ellipsoid<Scalar>>(
+        Ellipsoid<Scalar>(E_full_precision.cols()), from_parametrization,
+        from_parametrization.inverse());
 }
 
 }  // namespace samply
