@@ -41,7 +41,7 @@ public:
      * constraining the polytope.
      */
     Polytope(const Matrix<double>& G, const Matrix<double>& h)
-        : Polytope(G, h, AffineTransform<double>::identity(G.cols()))
+        : Polytope(G, h, AffineTransform<double>::identity(static_cast<Index>(G.cols())))
     {
     }
 
@@ -59,12 +59,12 @@ public:
      */
     Polytope(const Matrix<double>& G,
              const Matrix<double>& h,
-             const AffineTransform<double>& from_max_isotropy_frame_transform)
+             const AffineTransform<double>& from_optimal_frame_transform)
         : G_full_precision(G),
           h_full_precision(h),
           G_(G.cast<Scalar>()),
           h_(h.cast<Scalar>()),
-          from_max_isotropy_frame_transform_(from_max_isotropy_frame_transform),
+          from_optimal_frame_transform_(from_optimal_frame_transform),
           numerator_cache_(),
           denominator_cache_()
     {
@@ -88,7 +88,7 @@ public:
      * @return An object describing all the intersections of the rays with the
      * polytope.
      */
-    IntersectionsPacket<Scalar> intersect(const RaysPacket<Scalar>& rays);
+    IntersectionsPacket<Scalar> intersect(const AxisRaysPacket<Scalar>& rays);
 
     /**
      * @brief Find the intersections between a set of rays and the polytope.
@@ -98,7 +98,9 @@ public:
      * @return An object describing all the intersections of the rays with the
      * polytope.
      */
-    IntersectionsPacket<Scalar> intersect(const AxisRaysPacket<Scalar>& rays);
+    template <typename RaysPacketType>
+    IntersectionsPacket<Scalar> intersect(
+        const RaysPacketBase<ScalarType, RaysPacketType>& rays);
 
     /**
      * @brief Update the positions of the Markov Chains relative to the previous
@@ -109,6 +111,16 @@ public:
      */
     void update_position(const Vector<Scalar>& t);
 
+    /**
+     * @brief Get the dimensionality of the polytope.
+     *
+     * @return The dimensionality of the polytope.
+     */
+    Eigen::Index dimensionality() const
+    {
+        return G_.cols();
+    }
+
     /**
      * @brief Get a reparametrized version of this object such that intersection
      * operations are easy.
@@ -116,8 +128,8 @@ public:
      * @return Reparametrized version of this object, containing the
      * transforms mapping between the two parametrizations.
      */
-    ReparametrizedObject<Polytope<Scalar>> get_optimally_reparametrized_descriptor(
-        const std::string& optimality_criterion) const;
+    ReparametrizedObject<Polytope<Scalar>> get_optimally_reparametrized_descriptor()
+        const;
 
 private:
     // NumConstraints-by-NumDimensions matrix containing the normal
@@ -136,9 +148,8 @@ private:
     // constraining the polytope.
     const Vector<Scalar> h_;
 
-    // Transform from a coordinate frame in which the polytope has maximum
-    // isotropy to the frame in which G and h are given.
-    const AffineTransform<double> from_max_isotropy_frame_transform_;
+    // Transform from a coordinate frame in which intersection computations are easier.
+    const AffineTransform<double> from_optimal_frame_transform_;
 
     // Matrix caching the numerator and denominator used in the last
     // intersection tests.
@@ -156,44 +167,6 @@ void Polytope<ScalarType>::initialize(const Matrix<ScalarType>& state)
     numerator_cache_ = G_ * state - h_.replicate(1, state.cols());
 }
 
-template <typename ScalarType>
-IntersectionsPacket<ScalarType> Polytope<ScalarType>::intersect(
-    const RaysPacket<ScalarType>& rays)
-{
-    const Eigen::Index num_rays = rays.origins.cols();
-    Scalar infinity = std::numeric_limits<Scalar>::infinity();
-
-    // Find all intersections with the constraints. Note that if any
-    // element of G*directions is zero, NaNs will be generated. Since
-    // IEC 559 guarantees that <= and >= involving NaNs return false, these
-    // will be replaced by infinity when choosing the closest intersection
-    // later on.
-    denominator_cache_.noalias() = G_ * rays.directions;
-
-    auto all_intersections_t =
-        (Matrix<Scalar>(2 + h_.size(), num_rays)
-             << (-numerator_cache_.cwiseQuotient(denominator_cache_)),
-         RowVector<Scalar>::Ones(num_rays) * -infinity,
-         RowVector<Scalar>::Ones(num_rays) * infinity)
-            .finished();
-
-    // For each ray, find the first intersections in the positive and
-    // negative directions. For each direction, only select the elements
-    // with the correct sign and replace the others with +/- infinity.
-    IntersectionsPacket<Scalar> intersections(
-        (all_intersections_t.array() <= 0.0)
-            .select(all_intersections_t, -infinity)
-            .eval()
-            .colwise()
-            .maxCoeff(),
-        (all_intersections_t.array() >= 0.0)
-            .select(all_intersections_t, infinity)
-            .eval()
-            .colwise()
-            .minCoeff());
-    return intersections;
-}
-
 template <typename ScalarType>
 IntersectionsPacket<ScalarType> Polytope<ScalarType>::intersect(
     const AxisRaysPacket<ScalarType>& rays)
@@ -232,6 +205,45 @@ IntersectionsPacket<ScalarType> Polytope<ScalarType>::intersect(
     return intersections;
 }
 
+template <typename ScalarType>
+template <typename RaysPacketType>
+IntersectionsPacket<ScalarType> Polytope<ScalarType>::intersect(
+    const RaysPacketBase<ScalarType, RaysPacketType>& rays)
+{
+    const Eigen::Index num_rays = rays.origins.cols();
+    Scalar infinity = std::numeric_limits<Scalar>::infinity();
+
+    // Find all intersections with the constraints. Note that if any
+    // element of G*directions is zero, NaNs will be generated. Since
+    // IEC 559 guarantees that <= and >= involving NaNs return false, these
+    // will be replaced by infinity when choosing the closest intersection
+    // later on.
+    denominator_cache_.noalias() = G_ * rays.directions;
+
+    auto all_intersections_t =
+        (Matrix<Scalar>(2 + h_.size(), num_rays)
+             << (-numerator_cache_.cwiseQuotient(denominator_cache_)),
+         RowVector<Scalar>::Ones(num_rays) * -infinity,
+         RowVector<Scalar>::Ones(num_rays) * infinity)
+            .finished();
+
+    // For each ray, find the first intersections in the positive and
+    // negative directions. For each direction, only select the elements
+    // with the correct sign and replace the others with +/- infinity.
+    IntersectionsPacket<Scalar> intersections(
+        (all_intersections_t.array() <= 0.0)
+            .select(all_intersections_t, -infinity)
+            .eval()
+            .colwise()
+            .maxCoeff(),
+        (all_intersections_t.array() >= 0.0)
+            .select(all_intersections_t, infinity)
+            .eval()
+            .colwise()
+            .minCoeff());
+    return intersections;
+}
+
 template <typename ScalarType>
 void Polytope<ScalarType>::update_position(const Vector<Scalar>& t)
 {
@@ -240,51 +252,18 @@ void Polytope<ScalarType>::update_position(const Vector<Scalar>& t)
 
 template <typename ScalarType>
 ReparametrizedObject<Polytope<ScalarType>>
-Polytope<ScalarType>::get_optimally_reparametrized_descriptor(
-    const std::string& optimality_criterion) const
+Polytope<ScalarType>::get_optimally_reparametrized_descriptor() const
 {
-    if (optimality_criterion.compare("isotropy") == 0) {
-        return ReparametrizedObject<Polytope<Scalar>>(
-            Polytope<Scalar>(G_full_precision, h_full_precision,
-                             AffineTransform<double>::identity(G_.cols())),
-            from_max_isotropy_frame_transform_,
-            from_max_isotropy_frame_transform_.inverse());
-    } else if (optimality_criterion.compare("intersection") == 0) {
-        AffineTransform<double> from_parametrization =
-            AffineTransform<double>::identity(G_.cols());
-        return ReparametrizedObject<Polytope<Scalar>>(
-            Polytope<Scalar>(G_full_precision, h_full_precision,
-                             from_max_isotropy_frame_transform_),
-            from_parametrization, from_parametrization.inverse());
-    } else if (optimality_criterion.compare("isotropy-chrr") == 0) {
-        AffineTransform<double> from_parametrization =
-            from_max_isotropy_frame_transform_;
-        AffineTransform<double> to_parametrization =
-            from_max_isotropy_frame_transform_.inverse();
+    AffineTransform<double> from_parametrization = from_optimal_frame_transform_;
+    AffineTransform<double> to_parametrization =
+        from_optimal_frame_transform_.inverse();
 
-        return ReparametrizedObject<Polytope<Scalar>>(
-            Polytope<Scalar>(G_full_precision, h_full_precision,
-                             AffineTransform<double>::identity(G_.cols())),
-            from_parametrization, to_parametrization);
-    } else if (optimality_criterion.compare("intersection-chrr") == 0) {
-        AffineTransform<double> from_parametrization =
-            from_max_isotropy_frame_transform_;
-        AffineTransform<double> to_parametrization =
-            from_max_isotropy_frame_transform_.inverse();
-
-        return ReparametrizedObject<Polytope<Scalar>>(
-            Polytope<Scalar>(
-                G_full_precision * from_parametrization.get_linear(),
-                h_full_precision - G_full_precision * from_parametrization.get_shift(),
-                AffineTransform<double>::identity(G_.cols())),
-            from_parametrization, to_parametrization);
-    } else {
-        throw std::runtime_error(
-            std::string("The '") + optimality_criterion +
-            std::string("' "
-                        "optimality criterion is not recognized by the Ellipsoid "
-                        "descriptor"));
-    }
+    return ReparametrizedObject<Polytope<Scalar>>(
+        Polytope<Scalar>(
+            G_full_precision * from_parametrization.get_linear(),
+            h_full_precision - G_full_precision * from_parametrization.get_shift(),
+            AffineTransform<double>::identity(static_cast<Index>(G_.cols()))),
+        from_parametrization, to_parametrization);
 }
 
 }  // namespace samply
