@@ -21,15 +21,14 @@ namespace samply {
  * @brief Helper class for generating random elements.
  */
 class SamplingHelper {
-public:
+  public:
     /**
      * @brief Construct a new instance of the SamplingHelper class.
      */
     SamplingHelper()
         : generator(static_cast<unsigned int>(
               std::chrono::system_clock::now().time_since_epoch().count()))
-    {
-    }
+    {}
 
     /**
      * @brief Get a random scalar element within the specified bounds.
@@ -121,9 +120,9 @@ public:
      * @param num_values Number of elements to generate.
      * @return Vector containing the random integers.
      */
-    Vector<Eigen::Index> get_random_integers(const Eigen::Index min_value,
-                                             const Eigen::Index max_value,
-                                             const size_t num_values);
+    IndexVector get_random_integers(const Index min_value,
+                                    const Index max_value,
+                                    const size_t num_values);
 
     /**
      * @brief Sample an index given weights for each element.
@@ -159,8 +158,7 @@ public:
      * @param b Point at which the CDF must be computed.
      * @return The CDF at the given point.
      */
-    template <typename Scalar>
-    Scalar get_normal_cdf(const Scalar b);
+    template <typename Scalar> Scalar get_normal_cdf(const Scalar b);
 
     /**
      * @brief Compute the Cumulative Density Function (CDF) of a standard normal
@@ -175,7 +173,7 @@ public:
     Vector<Scalar> get_normal_cdf_interval(const Vector<Scalar>& a,
                                            const Vector<Scalar>& b);
 
-private:
+  private:
     // Type of the random engine used by the helper.
     typedef pcg32 Generator;
 
@@ -247,17 +245,18 @@ inline Matrix<Scalar> SamplingHelper::get_random_directions(const int num_dimens
     return random_normals * random_normals.colwise().norm().cwiseInverse().asDiagonal();
 }
 
-inline Vector<Eigen::Index> SamplingHelper::get_random_integers(
-    const Eigen::Index min_value, const Eigen::Index max_value, const size_t num_values)
+inline IndexVector SamplingHelper::get_random_integers(const Index min_value,
+                                                       const Index max_value,
+                                                       const size_t num_values)
 {
     std::uniform_int_distribution<> distribution(min_value, max_value);
-    return Vector<Eigen::Index>::NullaryExpr(num_values,
-                                             [&]() { return distribution(generator); });
+    return IndexVector::NullaryExpr(num_values,
+                                    [&]() { return distribution(generator); });
 }
 
 template <typename Scalar>
-inline Eigen::Index SamplingHelper::sample_index_with_weights(
-    const Vector<Scalar>& index_weights)
+inline Eigen::Index
+SamplingHelper::sample_index_with_weights(const Vector<Scalar>& index_weights)
 {
     std::discrete_distribution<Eigen::Index> distribution(index_weights.begin(),
                                                           index_weights.end());
@@ -274,8 +273,7 @@ inline Scalar SamplingHelper::get_random_truncated_normal(const Scalar mean,
         rtnorm(generator, lower_bound, upper_bound, mean, sigma));
 }
 
-template <typename Scalar>
-inline Scalar SamplingHelper::get_normal_cdf(const Scalar b)
+template <typename Scalar> inline Scalar SamplingHelper::get_normal_cdf(const Scalar b)
 {
     // https://en.cppreference.com/w/cpp/numeric/math/erfc
     return std::erfc(-b / std::sqrt(Scalar(2))) / Scalar(2);
@@ -289,6 +287,6 @@ inline Vector<Scalar> SamplingHelper::get_normal_cdf_interval(const Vector<Scala
            a.unaryExpr([this](const Scalar x) { return get_normal_cdf(x); });
 }
 
-}  // namespace samply
+} // namespace samply
 
 #endif
