@@ -1,4 +1,4 @@
-// Copyright (c) 2019 ETH Zurich, Mattia Gollub (mattia.gollub@bsse.ethz.ch)
+// Copyright (c) 2021 ETH Zurich, Mattia Gollub (mattia.gollub@bsse.ethz.ch)
 // Computational Systems Biology group, D-BSSE
 //
 // This software is freely available under the GNU General Public License v3.
@@ -34,26 +34,49 @@ namespace samply {
  * 1-dimensional chord at each step of the simulation.
  * @tparam Type of the logger used for monitoring the state of the simulation.
  */
-template <typename Scalar,
-          template <typename>
-          typename SpaceDescriptor,
-          template <typename>
-          typename ChordSampler,
-          template <typename, typename>
-          typename Logger>
-class HitAndRunSampler
-    : public MarkovChain<
-          HitAndRunSampler<Scalar, SpaceDescriptor, ChordSampler, Logger>,
-          Matrix<Scalar>,
-          Matrix<Scalar>,
-          Logger> {
-public:
+template <
+    typename Scalar,
+    template <typename>
+    typename DirectionSampler,
+    template <typename>
+    typename SpaceDescriptor,
+    template <typename>
+    typename ChordSampler,
+    template <typename, typename>
+    typename Logger>
+
+class HitAndRunSampler : public MarkovChain<
+                             HitAndRunSampler<
+                                 Scalar,
+                                 DirectionSampler,
+                                 SpaceDescriptor,
+                                 ChordSampler,
+                                 Logger>,
+                             Matrix<Scalar>,
+                             Matrix<Scalar>,
+                             Logger> {
+  public:
     /**
      * @brief Base class of the sampler providing general operations for Markov
      * Chains.
      */
     typedef MarkovChain<HitAndRunSampler, Matrix<Scalar>, Matrix<Scalar>, Logger> Base;
 
+    /**
+     * @brief The type of the class providing the direction sampler.
+     */
+    typedef DirectionSampler<Scalar> DirectionSamplerType;
+
+    /**
+     * @brief The type of the descriptor of the sampling space.
+     */
+    typedef SpaceDescriptor<Scalar> DescriptorType;
+
+    /**
+     * @brief The type of the 1D sampler.
+     */
+    typedef ChordSampler<Scalar> ChordSamplerType;
+
     // Make sure that the MarkovChain class can access the protected methods of
     // this, in particular the next_state() implementation.
     friend Base;
@@ -74,32 +97,30 @@ public:
      * @param logger Instance of the logger used to monitor the state of the
      * simulation.
      */
-    HitAndRunSampler(const SpaceDescriptor<Scalar>& descriptor,
-                     const ChordSampler<Scalar>& sampler,
-                     const typename Base::LoggerType& logger)
-        : Base(logger),
-          space_descriptor(descriptor),
-          chord_sampler(sampler),
-          space_descriptor_Fd(
-              descriptor.get_optimally_reparametrized_descriptor("isotropy")),
-          space_descriptor_Fi(
-              descriptor.get_optimally_reparametrized_descriptor("intersection")),
-          chord_sampler_Fs(sampler.get_optimally_reparametrized_sampler()),
-          transforms_(space_descriptor_Fd, space_descriptor_Fi, chord_sampler_Fs)
-    {
-    }
+    HitAndRunSampler(
+        const DirectionSampler<Scalar>& direction_sampler,
+        const SpaceDescriptor<Scalar>& descriptor,
+        const ChordSampler<Scalar>& sampler,
+        const typename Base::LoggerType& logger)
+        : Base(logger)
+        , direction_sampler(direction_sampler)
+        , space_descriptor(descriptor)
+        , chord_sampler(sampler)
+        , direction_sampler_Fp(
+              direction_sampler.get_optimally_reparametrized_descriptor())
+        , space_descriptor_Fi(descriptor.get_optimally_reparametrized_descriptor())
+        , chord_sampler_Fs(sampler.get_optimally_reparametrized_sampler())
+        , transforms_(direction_sampler_Fp, space_descriptor_Fi, chord_sampler_Fs)
+    {}
 
     const SpaceDescriptor<Scalar>& get_space_descriptor() const
     {
         return space_descriptor_Fi;
     }
 
-    const ChordSampler<Scalar>& get_chord_sampler() const
-    {
-        return chord_sampler_Fs;
-    }
+    const ChordSampler<Scalar>& get_chord_sampler() const { return chord_sampler_Fs; }
 
-protected:
+  protected:
     void initialize(const InternalState& state) override
     {
         space_descriptor_Fi.initialize(state);
@@ -113,22 +134,27 @@ protected:
      */
     InternalState next_state(const InternalState& state) override
     {
-        // 1) Sample new directions uniformly distributed in the Fd frame.
-        auto directions_Fd =
-            sampling_helper.get_random_directions<Scalar>(state.rows(), state.cols());
+        // 1) Sample new directions according to the direction sampler.
+        const auto directions_Fp{
+            direction_sampler_Fp.get_directions(static_cast<Index>(state.cols()))};
 
         // 2) Find the intersections between the current rays and the space
         //    descriptor in the Fi frame.
-        RaysPacket<Scalar> rays_Fi = {
-            state, transforms_.transform_directions_from_Fd_to_Fi(directions_Fd)};
+        typename DirectionSamplerType::RaysPacketType rays_Fi{
+            state, transforms_.transform_directions_from_Fp_to_Fi(directions_Fp)};
         const auto intersections = space_descriptor_Fi.intersect(rays_Fi);
 
         // 3) Sample new points along the resulting chords in the Fs frame. Then
         //    compute the new states in the internal frame (i.e. Fi).
-        RaysPacket<Scalar> rays_Fs =
-            transforms_.transform_rays_from_Fi_to_Fs(rays_Fi, directions_Fd);
-        Vector<Scalar> t =
-            chord_sampler_Fs.sample1d(rays_Fs, intersections, sampling_helper);
+        Vector<Scalar> t;
+        if (transforms_.is_Fs_equal_Fi()) {
+            t = chord_sampler_Fs.sample1d(rays_Fi, intersections, sampling_helper);
+        }
+        else {
+            const RaysPacket<Scalar> rays_Fs =
+                transforms_.transform_rays_from_Fi_to_Fs(rays_Fi, directions_Fp);
+            t = chord_sampler_Fs.sample1d(rays_Fs, intersections, sampling_helper);
+        }
         space_descriptor_Fi.update_position(t);
 
         return rays_Fi.at(t);
@@ -144,13 +170,16 @@ protected:
         return transforms_.transform_to_Fi(state);
     }
 
-private:
+  private:
     // Helper object used to generate random numbers.
     SamplingHelper sampling_helper;
 
+    // Sampler for ray directions.
+    DirectionSampler<Scalar> direction_sampler;
+    ReparametrizedObject<DirectionSampler<Scalar>> direction_sampler_Fp;
+
     // Descriptor of the sampling space.
     SpaceDescriptor<Scalar> space_descriptor;
-    ReparametrizedObject<SpaceDescriptor<Scalar>> space_descriptor_Fd;
     ReparametrizedObject<SpaceDescriptor<Scalar>> space_descriptor_Fi;
 
     // Object used for sampling on the 1-d chord of each hit-and-run step.
@@ -160,6 +189,6 @@ private:
     HitAndRunTransforms<Scalar> transforms_;
 };
 
-}  // namespace samply
+} // namespace samply
 
 #endif
