@@ -9,71 +9,93 @@
 
 #include <Eigen/Dense>
 
+#include "commons.h"
 #include "reparametrized_object.h"
 
 namespace samply {
 
-template <typename TransformationScalar>
-class HitAndRunTransforms {
-public:
+template <typename TransformationScalar> class HitAndRunTransforms {
+  public:
     // Fi is the coordinate system of the internal state.
     // If the chord sampler doesn't need a frame, Fs will be the same as Fi.
-    template <typename SpaceDescriptor, typename ChordSampler>
-    HitAndRunTransforms(const ReparametrizedObject<SpaceDescriptor>& descriptor_Fd,
-                        const ReparametrizedObject<SpaceDescriptor>& descriptor_Fi,
-                        const ReparametrizedObject<ChordSampler>& sampler_Fs)
-        : from_Fi(descriptor_Fi.from_frame().template cast<TransformationScalar>()),
-          to_Fi(descriptor_Fi.to_frame().template cast<TransformationScalar>()),
-          Fd_to_Fi_linear(
-              (descriptor_Fi.to_frame().linear() * descriptor_Fd.from_frame().linear())
-                  .template cast<TransformationScalar>()),
-          Fd_to_Fs_linear(
-              make_Fd_to_Fs_transform(descriptor_Fd, descriptor_Fi, sampler_Fs)),
-          Fi_to_Fs(make_Fi_to_Fs_transform(descriptor_Fi, sampler_Fs)),
-          prefer_Fd_to_Fs_over_Fi_to_Fs(
-              find_cheapest_linear_transform({Fi_to_Fs, Fd_to_Fs_linear}) == 1u),
-          Fs_equal_Fi(!sampler_Fs.needs_frame())
-    {
-    }
+    template <
+        typename DirectionSampler,
+        typename SpaceDescriptor,
+        typename ChordSampler>
+    HitAndRunTransforms(
+        const ReparametrizedObject<DirectionSampler>& direction_sampler_Fp,
+        const ReparametrizedObject<SpaceDescriptor>& descriptor_Fi,
+        const ReparametrizedObject<ChordSampler>& sampler_Fs)
+        : from_Fi(descriptor_Fi.from_frame().template cast<TransformationScalar>())
+        , to_Fi(descriptor_Fi.to_frame().template cast<TransformationScalar>())
+        , Fp_to_Fi_linear((descriptor_Fi.to_frame().linear() *
+                           direction_sampler_Fp.from_frame().linear())
+                              .template cast<TransformationScalar>())
+        , Fp_to_Fs_linear(
+              make_Fp_to_Fs_transform(direction_sampler_Fp, descriptor_Fi, sampler_Fs))
+        , Fi_to_Fs(make_Fi_to_Fs_transform(descriptor_Fi, sampler_Fs))
+        , prefer_Fp_to_Fs_over_Fi_to_Fs(
+              find_cheapest_linear_transform({Fi_to_Fs, Fp_to_Fs_linear}) == 1u)
+        , Fs_equal_Fi(!sampler_Fs.needs_frame())
+    {}
 
-    Matrix<TransformationScalar> transform_directions_from_Fd_to_Fi(
-        const Matrix<TransformationScalar>& directions_Fd)
+    template <typename DirectionsType>
+    DirectionsType
+    transform_directions_from_Fp_to_Fi(const DirectionsType& directions_Fp)
     {
-        return Fd_to_Fi_linear * directions_Fd;
+        return Fp_to_Fi_linear * directions_Fp;
     }
 
     RaysPacket<TransformationScalar> transform_rays_from_Fi_to_Fs(
         const RaysPacket<TransformationScalar>& rays_Fi,
-        const Matrix<TransformationScalar>& directions_Fd)
+        const Matrix<TransformationScalar>& directions_Fp)
     {
         if (Fs_equal_Fi) {
             return rays_Fi;
-        } else if (prefer_Fd_to_Fs_over_Fi_to_Fs) {
-            return RaysPacket<TransformationScalar>{Fi_to_Fs * rays_Fi.origins,
-                                                    Fd_to_Fs_linear * directions_Fd};
-        } else {
+        }
+        else if (prefer_Fp_to_Fs_over_Fi_to_Fs) {
+            return RaysPacket<TransformationScalar>{
+                Fi_to_Fs * rays_Fi.origins, Fp_to_Fs_linear * directions_Fp};
+        }
+        else {
             return Fi_to_Fs * rays_Fi;
         }
     }
 
-    Matrix<TransformationScalar> transform_from_Fi(
-        const Matrix<TransformationScalar>& origins_Fi)
+    RaysPacket<TransformationScalar> transform_rays_from_Fi_to_Fs(
+        const AxisRaysPacket<TransformationScalar>& rays_Fi,
+        const IndexVector& axis_ids)
+    {
+        if (Fs_equal_Fi) {
+            return AffineTransform<TransformationScalar>::identity(
+                       static_cast<Index>(rays_Fi.origins.rows())) *
+                   rays_Fi;
+        }
+        else if (prefer_Fp_to_Fs_over_Fi_to_Fs) {
+            return RaysPacket<TransformationScalar>{
+                Fi_to_Fs * rays_Fi.origins,
+                Fp_to_Fs_linear.get_linear()(Eigen::all, axis_ids)};
+        }
+        else {
+            return Fi_to_Fs * rays_Fi;
+        }
+    }
+
+    Matrix<TransformationScalar>
+    transform_from_Fi(const Matrix<TransformationScalar>& origins_Fi)
     {
         return from_Fi * origins_Fi;
     }
 
-    Matrix<TransformationScalar> transform_to_Fi(
-        const Matrix<TransformationScalar>& origins)
+    Matrix<TransformationScalar>
+    transform_to_Fi(const Matrix<TransformationScalar>& origins)
     {
         return to_Fi * origins;
     }
 
-    bool is_Fs_equal_Fi() const
-    {
-        return Fs_equal_Fi;
-    }
+    bool is_Fs_equal_Fi() const { return Fs_equal_Fi; }
 
-protected:
+  protected:
     size_t find_cheapest_linear_transform(
         const std::vector<
             AffineTransform<TransformationScalar>,
@@ -92,11 +114,11 @@ protected:
             return std::distance(transforms.begin(), optimal_transform_it);
 
         // If none is found, look for a diagonal transform.
-        optimal_transform_it =
-            std::find_if(transforms.begin(), transforms.end(),
-                         [](const AffineTransform<TransformationScalar>& transform) {
-                             return transform.get_linear().isDiagonal();
-                         });
+        optimal_transform_it = std::find_if(
+            transforms.begin(), transforms.end(),
+            [](const AffineTransform<TransformationScalar>& transform) {
+                return transform.get_linear().isDiagonal();
+            });
         if (optimal_transform_it != transforms.end())
             return std::distance(transforms.begin(), optimal_transform_it);
 
@@ -105,18 +127,21 @@ protected:
         return 0;
     }
 
-    template <typename SpaceDescriptor, typename ChordSampler>
-    AffineTransform<TransformationScalar> make_Fd_to_Fs_transform(
-        const ReparametrizedObject<SpaceDescriptor>& descriptor_Fd,
+    template <
+        typename DirectionSampler,
+        typename SpaceDescriptor,
+        typename ChordSampler>
+    AffineTransform<TransformationScalar> make_Fp_to_Fs_transform(
+        const ReparametrizedObject<DirectionSampler>& direction_sampler_Fp,
         const ReparametrizedObject<SpaceDescriptor>& descriptor_Fi,
         const ReparametrizedObject<ChordSampler>& sampler_Fs)
     {
         if (sampler_Fs.needs_frame())
             return (sampler_Fs.to_frame().linear() *
-                    descriptor_Fd.from_frame().linear())
+                    direction_sampler_Fp.from_frame().linear())
                 .template cast<TransformationScalar>();
         else
-            return descriptor_Fd.from_frame();
+            return direction_sampler_Fp.from_frame();
     }
 
     template <typename SpaceDescriptor, typename ChordSampler>
@@ -135,14 +160,14 @@ protected:
     const AffineTransform<TransformationScalar> from_Fi;
     const AffineTransform<TransformationScalar> to_Fi;
 
-    const AffineTransform<TransformationScalar> Fd_to_Fi_linear;
-    const AffineTransform<TransformationScalar> Fd_to_Fs_linear;
+    const AffineTransform<TransformationScalar> Fp_to_Fi_linear;
+    const AffineTransform<TransformationScalar> Fp_to_Fs_linear;
     const AffineTransform<TransformationScalar> Fi_to_Fs;
 
-    bool prefer_Fd_to_Fs_over_Fi_to_Fs;
+    bool prefer_Fp_to_Fs_over_Fi_to_Fs;
     bool Fs_equal_Fi;
 };
 
-}  // namespace samply
+} // namespace samply
 
 #endif
