@@ -9,38 +9,31 @@
 
 #include <Eigen/Dense>
 
+#include "Eigen/src/Core/util/Meta.h"
 #include "commons.h"
 
 namespace samply {
 
-template <typename Scalar>
-struct IntersectionsPacket {
+template <typename Scalar> struct IntersectionsPacket {
     Vector<Scalar> segment_starts;
     Vector<Scalar> segment_ends;
-    Eigen::VectorXi ray_indices;
+    samply::IndexVector ray_indices;
     Eigen::Index num_rays;
     bool one_ray_one_intersection;
 
     IntersectionsPacket(const Vector<Scalar>& segment_starts,
                         const Vector<Scalar>& segment_ends)
-        : segment_starts(segment_starts),
-          segment_ends(segment_ends),
-          num_rays(segment_starts.size()),
-          one_ray_one_intersection(true)
-    {
-    }
+        : segment_starts(segment_starts), segment_ends(segment_ends),
+          num_rays(segment_starts.size()), one_ray_one_intersection(true)
+    {}
 
     IntersectionsPacket(const Vector<Scalar>& segment_starts,
                         const Vector<Scalar>& segment_ends,
-                        const Eigen::VectorXi& ray_indices,
+                        const samply::IndexVector& ray_indices,
                         const Eigen::Index num_rays)
-        : segment_starts(segment_starts),
-          segment_ends(segment_ends),
-          ray_indices(ray_indices),
-          num_rays(num_rays),
-          one_ray_one_intersection(false)
-    {
-    }
+        : segment_starts(segment_starts), segment_ends(segment_ends),
+          ray_indices(ray_indices), num_rays(num_rays), one_ray_one_intersection(false)
+    {}
 
     template <typename IndexList>
     IntersectionsPacket<Scalar> operator()(const IndexList& ray_indices) const
@@ -51,8 +44,8 @@ struct IntersectionsPacket {
                                            segment_ends(ray_indices));
     }
 
-    IntersectionsPacket<Scalar> intersect(
-        const IntersectionsPacket<Scalar>& other) const
+    IntersectionsPacket<Scalar>
+    intersect(const IntersectionsPacket<Scalar>& other) const
     {
         if (!other.one_ray_one_intersection || !one_ray_one_intersection)
             throw std::runtime_error(
@@ -62,9 +55,8 @@ struct IntersectionsPacket {
         if (!((segment_ends.array() > other.segment_starts.array()) &&
               (other.segment_ends.array() > segment_starts.array()))
                  .all()) {
-            throw std::runtime_error(
-                "The intersections don't overlap on "
-                "at least one ray");
+            throw std::runtime_error("The intersections don't overlap on "
+                                     "at least one ray");
         }
 
         return IntersectionsPacket<Scalar>(
@@ -105,6 +97,6 @@ struct IntersectionsPacket {
     }
 };
 
-}  // namespace samply
+} // namespace samply
 
 #endif
\ No newline at end of file
