@@ -8,6 +8,8 @@
 #define SAMPLY_AFFINE_TRANSFORM_H
 
 #include <Eigen/QR>
+#include <exception>
+#include <stdexcept>
 
 #include "affine_dense_transform.h"
 #include "affine_diagonal_transform.h"
@@ -19,6 +21,7 @@
 #include "linear_transform.h"
 #include "rays_packet.h"
 #include "reset_transform.h"
+#include "row_selection_transform.h"
 #include "translation_transform.h"
 #include "zero_transform.h"
 
@@ -90,7 +93,8 @@ public:
      * @param rays Rays to which the transform must be applied.
      * @return The transformed rays.
      */
-    RaysPacket<Scalar> operator*(const RaysPacket<Scalar>& rays) const;
+    template <typename RaysStorage>
+    RaysPacket<Scalar> operator*(const RaysPacketBase<Scalar, RaysStorage>& rays) const;
 
     /**
      * @brief Apply this affine transform to a AxisRaysPacket object.
@@ -100,6 +104,15 @@ public:
      */
     RaysPacket<Scalar> operator*(const AxisRaysPacket<Scalar>& rays) const;
 
+    /**
+     * @brief Apply this affine transform to a vector of axis. Note that this is only
+     * valid if the transform is an identity transform.
+     *
+     * @param rays Axis to which the transform must be applied.
+     * @return The transformed axis.
+     */
+    IndexVector operator*(const Vector<Index>& axis) const;
+
     /**
      * @brief Returns a new affine transform containing only the linear part of
      * this transform.
@@ -119,13 +132,21 @@ public:
     template <typename CastScalar>
     AffineTransform<CastScalar> cast() const;
 
+    /**
+     * @brief Checks whether this transform represents an identity transform.
+     *
+     * @return true If the transform is an identity transform.
+     * @return false If the transform is any other transform.
+     */
+    bool is_identity() const;
+
     /**
      * @brief Create an identity affine transform of the specified size.
      *
      * @param n_dimensions Size of the identity transform.
      * @return The constructed identity transform.
      */
-    static AffineTransform<Scalar> identity(const Eigen::Index n_dimensions);
+    static AffineTransform<Scalar> identity(const Index n_dimensions);
 
 private:
     // Types of affine transforms.
@@ -135,6 +156,7 @@ private:
         Identity,
         Linear,
         LinearDiagonal,
+        RowSelection,
         Reset,
         Translation,
         Zero
@@ -146,6 +168,8 @@ private:
     // Recompute the type of this transform.
     void reset_transform_type_();
 
+    bool is_row_selection_();
+
     // Utilities casting this transform as an affine transform of a specific
     // type. This enables the usage of optimized operations provided by the
     // specialized transforms.
@@ -154,6 +178,7 @@ private:
     const auto asIdentity_() const noexcept;
     const auto asLinear_() const noexcept;
     const auto asLinearDiagonal_() const noexcept;
+    const auto asRowSelection_() const noexcept;
     const auto asReset_() const noexcept;
     const auto asTranslation_() const noexcept;
     const auto asZero_() const noexcept;
@@ -174,6 +199,8 @@ inline void AffineTransform<Scalar, Storage>::reset_transform_type_()
             type_ = TransformType_::Identity;
         else if (this->get_linear().isDiagonal())
             type_ = TransformType_::LinearDiagonal;
+        else if (is_row_selection_())
+            type_ = TransformType_::RowSelection;
         else
             type_ = TransformType_::Linear;
     } else {
@@ -189,6 +216,25 @@ inline void AffineTransform<Scalar, Storage>::reset_transform_type_()
     }
 }
 
+template <typename Scalar, typename Storage>
+inline bool AffineTransform<Scalar, Storage>::is_row_selection_()
+{
+    if (!this->get_shift().isZero())
+        return false;
+
+    if (!((this->get_linear().array() == 0.0) || (this->get_linear().array() == 1.0))
+             .all())
+        return false;
+
+    for (Eigen::Index i = 0; i < this->get_linear().rows(); ++i) {
+        auto row = this->get_linear().row(i);
+        if ((row.array() != 0.0).count() != 1)
+            return false;
+    }
+
+    return true;
+}
+
 template <typename Scalar, typename Storage>
 inline const auto AffineTransform<Scalar, Storage>::asAffineDiagonal_() const noexcept
 {
@@ -219,6 +265,12 @@ inline const auto AffineTransform<Scalar, Storage>::asLinearDiagonal_() const no
     return LinearDiagonalTransform<Scalar, StorageRef>(StorageRef(this->storage_));
 }
 
+template <typename Scalar, typename Storage>
+inline const auto AffineTransform<Scalar, Storage>::asRowSelection_() const noexcept
+{
+    return RowSelectionTransform<Scalar, StorageRef>(StorageRef(this->storage_));
+}
+
 template <typename Scalar, typename Storage>
 inline const auto AffineTransform<Scalar, Storage>::asReset_() const noexcept
 {
@@ -303,6 +355,9 @@ inline Matrix<Scalar> AffineTransform<Scalar, Storage>::operator*(
     case TransformType_::LinearDiagonal:
         return asLinearDiagonal_() * matrix;
         break;
+    case TransformType_::RowSelection:
+        return asRowSelection_() * matrix;
+        break;
     case TransformType_::Reset:
         return asReset_() * matrix;
         break;
@@ -316,8 +371,9 @@ inline Matrix<Scalar> AffineTransform<Scalar, Storage>::operator*(
 }
 
 template <typename Scalar, typename Storage>
+template <typename RaysStorage>
 inline RaysPacket<Scalar> AffineTransform<Scalar, Storage>::operator*(
-    const RaysPacket<Scalar>& rays) const
+    const RaysPacketBase<Scalar, RaysStorage>& rays) const
 {
     switch (type_) {
     case TransformType_::AffineDiagonal:
@@ -341,6 +397,10 @@ inline RaysPacket<Scalar> AffineTransform<Scalar, Storage>::operator*(
         return RaysPacket<Scalar>{asLinearDiagonal_() * rays.origins,
                                   asLinearDiagonal_() * rays.directions};
         break;
+    case TransformType_::RowSelection:
+        return RaysPacket<Scalar>{asRowSelection_() * rays.origins,
+                                  asRowSelection_() * rays.directions};
+        break;
     case TransformType_::Translation:
         return RaysPacket<Scalar>{asTranslation_() * rays.origins, rays.directions};
         break;
@@ -381,6 +441,9 @@ inline RaysPacket<Scalar> AffineTransform<Scalar, Storage>::operator*(
         return RaysPacket<Scalar>{asLinearDiagonal_() * rays.origins,
                                   this->get_linear()(Eigen::all, rays.axis_ids)};
         break;
+    case TransformType_::RowSelection:
+        throw std::runtime_error("Not implemented yet");
+        break;
     case TransformType_::Translation:
         return RaysPacket<Scalar>{asTranslation_() * rays.origins,
                                   this->get_linear()(Eigen::all, rays.axis_ids)};
@@ -396,6 +459,21 @@ inline RaysPacket<Scalar> AffineTransform<Scalar, Storage>::operator*(
     }
 }
 
+template <typename Scalar, typename Storage>
+inline IndexVector AffineTransform<Scalar, Storage>::operator*(
+    const IndexVector& axis) const
+{
+    switch (type_) {
+    case TransformType_::Identity:
+        return axis;
+        break;
+    default:
+        throw std::runtime_error(
+            "Axis vectors can only be transformed with an identity transform");
+        break;
+    }
+}
+
 template <typename Scalar, typename Storage>
 inline AffineTransform<Scalar> AffineTransform<Scalar, Storage>::linear() const
 {
@@ -427,13 +505,19 @@ inline AffineTransform<CastScalar> AffineTransform<Scalar, Storage>::cast() cons
         this->storage_.get_shift().template cast<CastScalar>());
 }
 
+template <typename Scalar, typename Storage>
+inline bool AffineTransform<Scalar, Storage>::is_identity() const
+{
+    return type_ == TransformType_::Identity;
+}
+
 //==============================================================================
 //	AffineTransform public static methods implementation.
 //==============================================================================
 
 template <typename Scalar, typename Storage>
 inline AffineTransform<Scalar> AffineTransform<Scalar, Storage>::identity(
-    const Eigen::Index n_dimensions)
+    const Index n_dimensions)
 {
     return AffineTransform<Scalar>(Matrix<Scalar>::Identity(n_dimensions, n_dimensions),
                                    Vector<Scalar>::Zero(n_dimensions));
