@@ -120,10 +120,21 @@ public:
         return diagonal_;
     }
 
+    /**
+     * @brief Gets the indices associated with the transform (may be unused).
+     *
+     * @return The indices associated with the linear transform.
+     */
+    const Vector<Eigen::Index>& get_indices() const noexcept
+    {
+        return indices_;
+    }
+
 protected:
     Matrix<Scalar> T_;
     Vector<Scalar> shift_;
     Vector<Scalar> diagonal_;
+    Vector<Eigen::Index> indices_;
 };
 
 /**
@@ -181,6 +192,16 @@ public:
         return storage_ref_.get_diagonal();
     }
 
+    /**
+     * @brief Gets the indices associated with the transform (may be unused).
+     *
+     * @return The indices associated with the linear transform.
+     */
+    const Vector<Eigen::Index>& get_indices() const noexcept
+    {
+        return storage_ref_.get_indices();
+    }
+
 private:
     AffineTransformStorage<Scalar>& storage_ref_;
 };
@@ -241,6 +262,16 @@ public:
         return storage_ref_.get_diagonal();
     }
 
+    /**
+     * @brief Gets the indices associated with the transform (may be unused).
+     *
+     * @return The indices associated with the linear transform.
+     */
+    const Vector<Eigen::Index>& get_indices() const noexcept
+    {
+        return storage_ref_.get_indices();
+    }
+
 private:
     const AffineTransformStorage<Scalar>& storage_ref_;
 };
@@ -252,28 +283,52 @@ private:
 template <typename ScalarType>
 inline AffineTransformStorage<ScalarType>::AffineTransformStorage(
     const Matrix<ScalarType>& T, const Vector<ScalarType>& shift)
-    : T_(T), shift_(shift), diagonal_(T.diagonal())
+    : T_(T),
+      shift_(shift),
+      diagonal_(T.diagonal()),
+      indices_(Vector<Eigen::Index>::Zero(shift.size()))
 {
+    for (Eigen::Index i = 0; i < T_.rows(); ++i) {
+        auto row = this->get_linear().row(i);
+        if ((row.array() != 0.0).count() == 1)
+            for (Eigen::Index j = 0; j < row.cols(); ++j) {
+                if (row(j) == 1.0) {
+                    indices_[i] = j;
+                    break;
+                }
+            }
+        else
+            indices_[i] = -1;
+    }
 }
 
 template <typename ScalarType>
 inline AffineTransformStorage<ScalarType>::AffineTransformStorage(
     const AffineTransformStorage<ScalarType>& other)
-    : T_(other.T_), shift_(other.shift_), diagonal_(other.diagonal_)
+    : T_(other.T_),
+      shift_(other.shift_),
+      diagonal_(other.diagonal_),
+      indices_(other.indices_)
 {
 }
 
 template <typename ScalarType>
 inline AffineTransformStorage<ScalarType>::AffineTransformStorage(
     const AffineTransformStorageRef<ScalarType>& other)
-    : T_(other.T_), shift_(other.shift_), diagonal_(other.diagonal_)
+    : T_(other.T_),
+      shift_(other.shift_),
+      diagonal_(other.diagonal_),
+      indices_(other.indices_)
 {
 }
 
 template <typename ScalarType>
 inline AffineTransformStorage<ScalarType>::AffineTransformStorage(
     const AffineTransformStorageConstRef<ScalarType>& other)
-    : T_(other.T_), shift_(other.shift_), diagonal_(other.diagonal_)
+    : T_(other.T_),
+      shift_(other.shift_),
+      diagonal_(other.diagonal_),
+      indices_(other.indices_)
 {
 }
 
@@ -282,7 +337,8 @@ inline AffineTransformStorage<ScalarType>::AffineTransformStorage(
     AffineTransformStorage<ScalarType>&& other)
     : T_(std::move(other.T_)),
       shift_(std::move(other.shift_)),
-      diagonal_(std::move(other.diagonal_))
+      diagonal_(std::move(other.diagonal_)),
+      indices_(std::move(other.indices_))
 {
 }
 
