@@ -17,13 +17,7 @@ class Metabolite:
         Charge of the metabolite, by default 0.
     """
 
-    def __init__(
-        self,
-        key: str,
-        compartment: str = 'c',
-        nH: int = 0,
-        z: int = 0
-    ):
+    def __init__(self, key: str, compartment: str = "c", nH: int = 0, z: int = 0):
         self._key = key
         self._compartment = compartment
         self._nH = nH
