@@ -6,30 +6,40 @@ from .metabolite import Metabolite
 
 
 class CompartmentConvention(Enum):
-    """The convention used to define the compartment of a metabolite.
-    """
+    """The convention used to define the compartment of a metabolite."""
+
     UNDERSCORE = auto()
     """The metabolite ID ends with "_<compartment_ID>" (e.g. g6p_c).
     """
 
+    ASSUME_CYTOSOL = auto()
+    """The compartment is not specified, all metabolites are assumed to belong to the
+    cytosol.
+    """
+
 
 class Namespace(Enum):
-    """The namespace in which metabolite identifiers are defined.
-    """
+    """The namespace in which metabolite identifiers are defined."""
+
     UNSPECIFIED = auto()
     """The namespace is not specified. In most cases PTA can find the metabolite
-    in eQuilbrator even if the namespace is not specified. However, this is not
+    in eQuilibrator even if the namespace is not specified. However, this is not
     guaranteed and it is recommended to always specify a namespace.
     """
     BIGG = auto()
     """Assume that metabolite identifiers belong to the BiGG namespace.
     """
+    NONE = auto()
+    """The identifiers do not represent actual metabolites.
+    """
 
 
 class MetaboliteInterpreter:
     """
     An interpreter used to transform metabolite IDs to compound identifiers that can be
-    understood by eQuilibrator and compartment identifiers.
+    understood by eQuilibrator and compartment identifiers. Note: this class
+    could/should partially replaced by cobrapy's functionalities if/when this feature is
+    implemented: https://github.com/opencobra/cobrapy/issues/967
 
     Parameters
     ----------
@@ -43,19 +53,15 @@ class MetaboliteInterpreter:
 
     def __init__(
         self,
-        compartment_convention: CompartmentConvention =
-            CompartmentConvention.UNDERSCORE,
-        namespace: Namespace = Namespace.BIGG
+        compartment_convention: CompartmentConvention = CompartmentConvention.UNDERSCORE,
+        namespace: Namespace = Namespace.BIGG,
     ):
         self.compartment_convention = compartment_convention
         self.namespace = namespace
 
-    def read(
-        self,
-        metabolite_name: str
-    ) -> Metabolite:
+    def read(self, metabolite_name: str) -> Metabolite:
         """Parses a metabolite from a string, reading its identifier and
-        compartment. 
+        compartment.
 
         Parameters
         ----------
@@ -75,21 +81,31 @@ class MetaboliteInterpreter:
             If the naming convention or namespace of the metabolite are not supported.
         """
 
-        metabolite_id = ''
-        compartment = ''
+        metabolite_id = ""
+        compartment = ""
 
         if self.compartment_convention == CompartmentConvention.UNDERSCORE:
-            (metabolite_id, compartment) = metabolite_name.rsplit('_', 1)
+            assert "_" in metabolite_name, (
+                "Metabolite identifier does not appear to follow the UNDERSCORE "
+                "convention. You may need to specify a different convention."
+            )
+            (metabolite_id, compartment) = metabolite_name.rsplit("_", 1)
+        elif self.compartment_convention == CompartmentConvention.ASSUME_CYTOSOL:
+            metabolite_id = metabolite_name
+            compartment = "c"
         else:
-            raise Exception(f'Unsupported compartment convention: '
-                            f'{self.compartment_convention}')
+            raise Exception(
+                f"Unsupported compartment convention: {self.compartment_convention}"
+            )
 
         if self.namespace == Namespace.UNSPECIFIED:
             pass
         elif self.namespace == Namespace.BIGG:
-            metabolite_id = 'bigg.metabolite:' + metabolite_id
+            metabolite_id = "bigg.metabolite:" + metabolite_id
+        elif self.namespace == Namespace.NONE:
+            metabolite_id = "pseudometabolite:" + metabolite_id
         else:
-            raise Exception(f'Unsupported namespace: {self.namespace}')
+            raise Exception(f"Unsupported namespace: {self.namespace}")
 
         return Metabolite(metabolite_id, compartment)
 
