@@ -4,14 +4,10 @@
 import logging
 from typing import List, Optional, Union
 
+import cobra
 import numpy as np
 import pandas as pd
-
-import cobra
-from cobra.flux_analysis.variability import (
-    find_blocked_reactions,
-    flux_variability_analysis,
-)
+from cobra.flux_analysis.variability import find_blocked_reactions
 
 from .constants import (
     Q,
@@ -77,7 +73,7 @@ class StructuralAssessment:
         print("---------------------------------------------")
         print("")
 
-        pd_context = pd.option_context(
+        _ = pd.option_context(
             "display.max_rows",
             None,
             "display.max_columns",
@@ -102,7 +98,7 @@ class StructuralAssessment:
             for i, cycle in enumerate(self.forced_internal_cycles):
                 print(f'{i}. {", ".join(cycle)}')
         else:
-            print(f"> No thermodynamic inconsistencies detected in the model.")
+            print("> No thermodynamic inconsistencies detected in the model.")
         print("")
 
     def autoresolve(
@@ -142,7 +138,7 @@ class StructuralAssessment:
                 for rxn_id in cycle:
                     try:
                         reaction = model.reactions.get_by_id(rxn_id)
-                    except:
+                    except Exception:
                         continue
                     reaction.lower_bound = -default_bound
                     reaction.upper_bound = default_bound
@@ -156,7 +152,7 @@ class StructuralAssessment:
             # boundaries.
             try:
                 fixed_reaction = model.reactions.get_by_id(cycle[0])
-            except:
+            except Exception:
                 # If the reaction does not exist, the cycle is broken and we don't need
                 # to do anything else.
                 return False
@@ -170,10 +166,10 @@ class StructuralAssessment:
 
             # If all other reactions of the cycle are not fixed as well, then this is a
             # dead-end cycle.
-            for i in range(len(cycle)):
+            for rxn_id in cycle:
                 try:
-                    other_reaction = model.reactions.get_by_id(cycle[i])
-                except:
+                    other_reaction = model.reactions.get_by_id(rxn_id)
+                except Exception:
                     return False
                 model.objective = {other_reaction: 1}
                 max_flux = model.slim_optimize()
@@ -205,7 +201,7 @@ class StructuralAssessment:
         rxn_idxs = np.where(cycle != 0)[0].tolist()
 
         # Check for any pair of reaction and for any sense of the cycle.
-        for i in range(len(rxn_idxs)):
+        for i, _ in enumerate(rxn_idxs):
             for j in range(i):
                 for sense in [-1, 1]:
                     with model as model:
@@ -358,7 +354,7 @@ class QuantitativeAssessment:
 
         print("Quantitative thermodynamic assessment summary:")
         print("------------------------------------------------")
-        print("conentrations: mM, free energies: kJ/mol")
+        print("concentrations: mM, free energies: kJ/mol")
         print("")
 
         pd_context = pd.option_context(
@@ -386,7 +382,7 @@ class QuantitativeAssessment:
             with pd_context:
                 print(anomalies_m_z)
         else:
-            print(f"> No anomaly found in concentration z-scores.")
+            print("> No anomaly found in concentration z-scores.")
         print("")
 
         # Search for anomalies based on maximum concentration.
@@ -408,7 +404,7 @@ class QuantitativeAssessment:
             with pd_context:
                 print(anomalies_m_m)
         else:
-            print(f"> No anomaly found in non-intracellular concentrations.")
+            print("> No anomaly found in non-intracellular concentrations.")
         print("")
 
         # Search for anomalies based on reaction z-scores.
@@ -432,7 +428,7 @@ class QuantitativeAssessment:
             with pd_context:
                 print(anomalies_r_z)
         else:
-            print(f"> No anomaly found in free energy z-scores.")
+            print("> No anomaly found in free energy z-scores.")
         print("")
 
         # Search for anomalies based on reaction shadow prices.
@@ -449,7 +445,7 @@ class QuantitativeAssessment:
             with pd_context:
                 print(anomalies_r_s)
         else:
-            print(f"> No significant thermodynamic constraints found.")
+            print("> No significant thermodynamic constraints found.")
         print("")
 
     def _make_z_scores(
@@ -467,7 +463,7 @@ class QuantitativeAssessment:
             if pmo_problem.B.to_drg0_transform is not None:
                 transform = pmo_problem.B.to_drg0_transform[0]
         else:
-            raise ValueError(f"Unsuppoted variable {variable}")
+            raise ValueError(f"Unsupported variable {variable}")
 
         if transform is None:
             return np.nan
@@ -487,6 +483,7 @@ def prepare_for_pta(
     autoresolve_inconsistencies: bool = True,
     remove_blocked_reactions: bool = True,
     tighten_bounds: bool = True,
+    prevent_loops: bool = False,
 ):
     """Attempt to automatically prepare a model for use in PTA.
 
@@ -518,6 +515,7 @@ def prepare_for_pta(
         True if the method should restrict the flux bounds with FVA, false otherwise. By
         default True.
     """
+    logger.info("Preparing model for PTA ...")
     # Run structural assessment and auto-resolve all inconsistencies.
     if autoresolve_inconsistencies:
         assessment = StructuralAssessment(model, biomass_id, atpm_id)
@@ -530,4 +528,5 @@ def prepare_for_pta(
 
     # Restrict reaction bounds using FVA.
     if tighten_bounds:
-        tighten_model_bounds(model)
+        tighten_model_bounds(model, prevent_loops=prevent_loops)
+    logger.info("Model preprocessing completed.")
