@@ -1,5 +1,5 @@
-import copy
-import datetime
+"""Thermodynamics-based sampling of free energies, concentrations and fluxes."""
+
 import logging
 import math
 import multiprocessing
@@ -9,28 +9,28 @@ import random
 import tempfile
 from typing import List, Optional, Tuple, Union
 
+import _pta_python_binaries as pb
+import cobra
 import cvxpy as cp
 import numpy as np
 import pandas as pd
+from pta.sampling.convergence_manager import ConvergenceManager
 from scipy.stats.distributions import chi2
 
-import _pta_python_binaries as pb
-import cobra
-
 from ..constants import (
     default_max_psrf,
+    default_max_threads,
     default_min_eigenvalue_tds_basis,
     default_num_samples,
     tfs_default_feasibility_cache_size,
     tfs_default_min_rel_region_length,
-    tfs_steps_multiplier,
 )
 from ..flux_space import FluxSpace
 from ..pmo import PmoProblem, PmoProblemPool
 from ..thermodynamic_space import ThermodynamicSpace, ThermodynamicSpaceBasis
 from ..utils import apply_transform, covariance_square_root
 from .commons import (
-    SamplingException,
+    SamplerInterface,
     SamplingResult,
     apply_to_chains,
     fill_common_sampling_settings,
@@ -44,7 +44,7 @@ logger = logging.getLogger(__name__)
 _us_model: cobra.Model
 
 
-class TFSModel:
+class TFSModel(SamplerInterface):
     """Object holding the information necessary to run TFS.
 
     Parameters
@@ -83,12 +83,15 @@ class TFSModel:
         min_drg: float = 1e-1,
         max_drg: float = 1000,
         solver: Optional[str] = None,
-        solver_options: dict = {},
+        solver_options: dict = None,
     ):
+        # Create the flux space and its basis if needed.
         if isinstance(network, FluxSpace):
             self._F = network.copy()
         else:
             self._F = FluxSpace.from_cobrapy_model(network)
+
+        # Create the basis of the thermodynamic space if needed.
         if thermodynamic_space_basis is not None:
             self._B = thermodynamic_space_basis
             assert (
@@ -114,14 +117,9 @@ class TFSModel:
             min_drg,
             max_drg,
             solver,
-            solver_options,
+            solver_options or {},
         ]
 
-    @property
-    def B(self) -> ThermodynamicSpaceBasis:
-        """Gets the basis of the thermodynamic space used for sampling."""
-        return self._B
-
     @property
     def T(self) -> ThermodynamicSpace:
         """Gets the thermodynamic space used for sampling."""
@@ -132,6 +130,11 @@ class TFSModel:
         """Gets the flux space used for sampling."""
         return self._F
 
+    @property
+    def B(self) -> ThermodynamicSpaceBasis:
+        """Gets the basis of the thermodynamic space used for sampling."""
+        return self._B
+
     @property
     def pmo_args(self):
         """Gets the arguments used to construct PMO problems."""
@@ -163,54 +166,166 @@ class TFSModel:
             if self.F.lb[i] < 0 and self.F.ub[i] > 0 and id in self.T.reaction_ids
         ]
 
-    def to_drg(self, value: np.array) -> np.array:
+    def to_drg(self, value: np.ndarray) -> np.ndarray:
         """Transform a point or matrix from the the basis to reaction energies.
 
         Parameters
         ----------
-        value : np.array
+        value : np.ndarray
             Input values in the basis.
 
         Returns
         -------
-        np.array
+        np.ndarray
             The corresponding reaction energies.
         """
         assert self.B.to_drg_transform is not None
         return apply_transform(value, self.B.to_drg_transform)
 
-
-class FreeEnergiesSamplingResult(SamplingResult):
-    """Encapsulates the result of sampling reaction energies.
-
-    Parameters
-    ----------
-    samples : pd.DataFrame
-        Data frame containing the free energy samples.
-    psrf : pd.Series
-        The Potential Scale Reduction Factors of each variable.
-    orthants : pd.DataFrame
-        Data frame containing the signs of the reversible reactions for each orthants.
-        Contains an additional column ("weight") describing the weight of the orthant.
-    """
-
-    def __init__(
+    def simulate(
         self,
-        samples: pd.DataFrame,
-        psrf: pd.Series,
-        orthants: pd.DataFrame,
-    ):
-        SamplingResult.__init__(self, samples, psrf)
-        self._orthants = orthants
+        settings: pb.SamplerSettings,
+        initial_points: np.ndarray,
+        directions_transform: np.ndarray,
+    ) -> pb.TFSResult:
+        """Run the sampler with the given parameters.
 
-    @property
-    def orthants(self) -> pd.DataFrame:
-        """Gets a data frame containing the sampled orthants. Contains an additional
-        column ("weight") describing the weight of the orthant."""
-        return self._orthants
+        Parameters
+        ----------
+        settings : pb.SamplerSettings
+            Sampling settings.
+        initial_points : np.ndarray
+            The initial points for the chains.
+        directions_transform : np.ndarray
+            The transform for the directions sampler.
+        """
+        drg_G, drg_h = _make_drg_polytope(self)
+        result = pb.sample_free_energies(
+            self.F.S,
+            self.F.lb,
+            self.F.ub,
+            np.array(
+                [[self.F.reaction_ids.index(id) for id in self.T.reaction_ids]],
+                dtype=np.uint32,
+            ).T,
+            initial_points,
+            settings,
+            drg_G,
+            drg_h,
+            self.B.to_drg_transform[0],
+            self.B.to_drg_transform[1],
+            directions_transform,
+        )
+        return result
+
+    def compute_psrf(self, result: pb.TFSResult) -> pd.Series:
+        """Compute the potential scale reduction factors for the variables of interest
+        on a given set of chains.
+
+        Parameters
+        ----------
+        result : pb.TFSResult
+            The result of the sampling function.
+
+        Returns
+        -------
+        pd.Series
+            The computed potential scale reduction factors.
+        """
+
+        logger.debug("Computing PSRFs.")
+        basis_var_names = ["drg_var" + str(i) for i in range(self.B.dimensionality)]
+        var_names = [id + "_drg" for id in self.T.reaction_ids]
+        psrf_var_names = basis_var_names + var_names
+        psrf = np.hstack(
+            [
+                split_R(result.chains),
+                split_R(apply_to_chains(result.chains, self.to_drg)),
+            ]
+        )
+        return pd.Series(psrf, index=psrf_var_names)
+
+    def get_chains(self, result: pb.TFSResult) -> np.ndarray:
+        """Extract the simulated chains from a given result.
+
+        Parameters
+        ----------
+        result : pb.TFSResult
+            The result of the native sampling function.
+
+        Returns
+        -------
+        np.ndarray
+            The simulated chains.
+        """
+
+        return result.chains
+
+    def get_initial_points(self, num_points: int) -> np.ndarray:
+        """Gets initial points for sampling reaction energies.
+
+        Parameters
+        ----------
+        num_points : int
+            Number of initial points to generate.
+
+        Returns
+        -------
+        np.ndarray
+            Array containing the initial points.
+        """
+        # pylint: disable=protected-access
+        # Create the process pool for solving PMO problems.
+        pool = PmoProblemPool(None, *self._pmo_args)
+
+        # Find candidate optimization direactions.
+        reaction_idxs_T = list(range(len(self.T.reaction_ids)))
+        reaction_idxs_F = [self.F.reaction_ids.index(id) for id in self.T.reaction_ids]
+        only_forward_ids_T = [
+            i for i in reaction_idxs_T if self.F.lb[reaction_idxs_F[i]] >= 0
+        ]
+        only_backward_ids_T = [
+            i for i in reaction_idxs_T if self.F.ub[reaction_idxs_F[i]] <= 0
+        ]
+        reversible_ids_T = [
+            i
+            for i in reaction_idxs_T
+            if self.F.lb[reaction_idxs_F[i]] < 0 and self.F.ub[reaction_idxs_F[i]] > 0
+        ]
+
+        reversible_dirs = [(i, -1) for i in reversible_ids_T] + [
+            (i, 1) for i in reversible_ids_T
+        ]
+        irreversible_dirs = [(i, -1) for i in only_backward_ids_T] + [
+            (i, 1) for i in only_forward_ids_T
+        ]
+
+        # Select optimization directions, giving precedence to the reversible reactions.
+        if num_points >= len(reversible_dirs):
+            directions = reversible_dirs
+            directions_pool = irreversible_dirs
+            to_sample = min(num_points - len(reversible_dirs), len(irreversible_dirs))
+        else:
+            directions = []
+            directions_pool = reversible_dirs
+            to_sample = min(num_points, len(reversible_dirs))
+        optimization_directions = directions + random.sample(directions_pool, to_sample)
+
+        # Run the optimizations in the pool.
+        initial_points = pool.map(_find_point, optimization_directions)
+        assert all(p is not None for p in initial_points), (
+            "One or more initial points could not be found. This could be due to "
+            "an overconstrained model or numerical inaccuracies."
+        )
+
+        points_array = np.hstack(initial_points)
+        pool.close()
+
+        return points_array
 
 
-def _find_point(problem: PmoProblem, flux_objective: Tuple[int, int]) -> np.array:
+def _find_point(problem: PmoProblem, flux_objective: Tuple[int, int]) -> np.ndarray:
+    # pylint: disable=protected-access
     # First, find an orthant with the given direction (if possible).
     problem.objective = lambda p: cp.Maximize(
         flux_objective[1] * p.d[flux_objective[0]]
@@ -268,81 +383,58 @@ def _find_point(problem: PmoProblem, flux_objective: Tuple[int, int]) -> np.arra
         return None
     else:
         logger.info(f"Found initial point with distance {distance.value}")
-        return orthant_problem.drg
+        return orthant_problem.m.value
 
 
-def get_initial_points(model: TFSModel, num_points: int) -> np.array:
-    """Gets initial points for sampling reaction energies.
+class FreeEnergiesSamplingResult(SamplingResult):
+    """Encapsulates the result of sampling reaction energies.
 
     Parameters
     ----------
-    model : TFSModel
-        The model to sample.
-    num_points : int
-        Number of initial points to generate.
-
-    Returns
-    -------
-    np.array
-        Array containing the initial points.
+    samples : pd.DataFrame
+        Data frame containing the free energy samples.
+    psrf : pd.Series
+        The Potential Scale Reduction Factors of each variable.
+    orthants : pd.DataFrame
+        Data frame containing the signs of the reversible reactions for each orthants.
+        Contains an additional column ("weight") describing the weight of the orthant.
+    basis_samples : pd.DataFrame, optional
+        The samples in the basis.
+    chains : np.ndarray, optional
+        The simulated chains.
     """
 
-    # Create the process pool for solving PMO problems.
-    pool = PmoProblemPool(None, *model._pmo_args)
+    def __init__(
+        self,
+        samples: pd.DataFrame,
+        psrf: pd.Series,
+        orthants: pd.DataFrame,
+        basis_samples: pd.DataFrame = None,
+        chains: np.ndarray = None,
+    ):
+        SamplingResult.__init__(self, samples, psrf, basis_samples, chains)
+        self._orthants = orthants
 
-    # Find candidate optimization direactions.
-    reaction_idxs_T = list(range(len(model.T.reaction_ids)))
-    reaction_idxs_F = [model.F.reaction_ids.index(id) for id in model.T.reaction_ids]
-    only_forward_ids_T = [
-        i for i in reaction_idxs_T if model.F.lb[reaction_idxs_F[i]] >= 0
-    ]
-    only_backward_ids_T = [
-        i for i in reaction_idxs_T if model.F.ub[reaction_idxs_F[i]] <= 0
-    ]
-    reversible_ids_T = [
-        i
-        for i in reaction_idxs_T
-        if model.F.lb[reaction_idxs_F[i]] < 0 and model.F.ub[reaction_idxs_F[i]] > 0
-    ]
-
-    reversible_dirs = [(i, -1) for i in reversible_ids_T] + [
-        (i, 1) for i in reversible_ids_T
-    ]
-    irreversible_dirs = [(i, -1) for i in only_backward_ids_T] + [
-        (i, 1) for i in only_forward_ids_T
-    ]
-
-    # Select optimization directions, giving precedence to the reversible reactions.
-    if num_points >= len(reversible_dirs):
-        directions = reversible_dirs
-        directions_pool = irreversible_dirs
-        to_sample = min(num_points - len(reversible_dirs), len(irreversible_dirs))
-    else:
-        directions = []
-        directions_pool = reversible_dirs
-        to_sample = min(num_points, len(reversible_dirs))
-    optimization_directions = random.sample(directions_pool, to_sample)
-
-    # Run the optimizations in the pool.
-    initial_points = pool.map(_find_point, optimization_directions)
-    points_array = np.hstack(initial_points)
-    pool.close()
-
-    return points_array
+    @property
+    def orthants(self) -> pd.DataFrame:
+        """Gets a data frame containing the sampled orthants. Contains an additional
+        column ("weight") describing the weight of the orthant."""
+        return self._orthants
 
 
 def sample_drg(
     model: TFSModel,
     num_samples: int = default_num_samples,
     num_direction_samples: int = default_num_samples,
-    initial_points: np.array = None,
-    num_steps: int = -1,
-    num_warmup_steps: int = -1,
-    num_chains: int = -1,
-    log_interval: datetime.timedelta = None,
+    max_steps: int = -1,
     max_psrf: float = default_max_psrf,
+    num_chains: int = -1,
+    initial_points: np.ndarray = None,
+    num_initial_steps: int = -1,
     feasibility_cache_size: int = tfs_default_feasibility_cache_size,
     min_rel_region_length: float = tfs_default_min_rel_region_length,
+    max_threads: int = default_max_threads,
+    convergence_manager: ConvergenceManager = None,
 ) -> FreeEnergiesSamplingResult:
     """Sample reaction energies under steady state flux constraints in the given model.
 
@@ -354,24 +446,26 @@ def sample_drg(
         Number of samples to draw.
     num_direction_samples : int, optional
         Number of orthant samples to collect.
-    initial_points : np.array, optional
-        The initial points for the chains.
-    num_steps : int, optional
-        The number of steps to use for each chain.
-    num_warmup_steps : int, optional
-        The number of burn-in steps to discard.
-    num_chains : int, optional
-        The number of chains to simulate.
-    log_interval : datetime.timedelta, optional
-        How often should the sampler log the progress.
+    max_steps : int, optional
+        The maximum number fo steps to simulate.
     max_psrf : float, optional
         Maximum value of the PSRFs for convergence.
+    num_chains : int, optional
+        The number of chains to simulate.
+    initial_points : np.ndarray, optional
+        The initial points for the chains.
+    num_initial_steps: int, optional
+        Initial chains length.
     feasibility_cache_size : int, optional
         Maximum size of the cache storing the feasibility of the orthants encountered
         during the random walk.
     min_rel_region_length : float, optional
         Minimum length (relative to the length of the entire ray) of a segment in order
         to consider it for sampling.
+    max_threads : int, optional
+        The maximum number of parallel threads to use.
+    convergence_manager : ConvergenceManager, optional
+        The object to use to monitor and improve convergence.
 
     Returns
     -------
@@ -383,27 +477,102 @@ def sample_drg(
     SamplingException
         If sampling fails.
     """
-    if num_steps < 0:
-        num_steps = model.dimensionality ** 2 * tfs_steps_multiplier
-    if initial_points is not None and num_chains < 0:
-        num_chains = initial_points.shape[1]
-
-    # Fill the settings.
-    settings = pb.FreeEnergySamplerSettings()
-    fill_common_sampling_settings(
-        settings,
-        "",
-        num_samples,
-        num_steps,
-        num_chains,
-        num_warmup_steps,
-        log_interval,
+    # Validate and process input settings.
+    assert model.B.to_drg_transform is not None
+    assert num_samples > 0
+    assert num_direction_samples > 0
+    max_steps = max_steps if max_steps >= 1 else 2 ** 63
+    assert max_psrf > 1.0
+    if num_chains < 0:
+        if initial_points is not None:
+            num_chains = initial_points.shape[1]
+        else:
+            num_chains = max(model.F.n_reversible_reactions * 2, 1)
+    num_initial_steps = (
+        num_initial_steps
+        if num_initial_steps > 0
+        else model.B.dimensionality ** 2 + num_samples
     )
+    assert feasibility_cache_size >= 0
+    assert min_rel_region_length > 0
+    assert max_threads > 0
+    if convergence_manager is None:
+        convergence_manager = ConvergenceManager(model, num_initial_steps, True)
+
+    # Construct settings and auxiliary functions.
+    settings = pb.FreeEnergySamplerSettings()
     settings.truncation_multiplier = model.confidence_radius
     settings.feasibility_cache_size = feasibility_cache_size
     settings.drg_epsilon = model.drg_epsilon
     settings.flux_epsilon = 1e-4  # TODO: make this an actual setting.
     settings.min_rel_region_length = min_rel_region_length
+    update_settings_function = lambda s, steps: _fill_settings(
+        s, num_samples, steps, num_chains, 2, max_threads, num_direction_samples
+    )
+    update_settings_function(settings, num_initial_steps)
+    make_sampling_result_function = lambda r: _make_sampling_result(
+        model, r, num_samples
+    )
+
+    # Generate the initial points if needed.
+    if initial_points is None:
+        initial_points = model.get_initial_points(settings.num_chains)
+
+    # Run the sampler.
+    result = convergence_manager.run(
+        settings,
+        update_settings_function,
+        make_sampling_result_function,
+        initial_points,
+        max_steps,
+        max_psrf,
+    )
+
+    return result
+
+
+def _make_sampling_result(
+    model: "TFSModel", result: pb.TFSResult, num_samples: int
+) -> FreeEnergiesSamplingResult:
+    basis_var_names = ["var" + str(i) for i in range(model.dimensionality)]
+    basis_samples = sample_from_chains(result.chains, num_samples)
+    samples = model.to_drg(basis_samples.T).T
+
+    # Convert the orthants from binary to integer description.
+    orthant_signs = np.unpackbits(result.directions, bitorder="little", axis=1)[
+        :, : len(model.reversible_rxn_ids)
+    ].astype(np.int8)
+    orthant_signs[orthant_signs == 0] = -1
+
+    orthants = pd.DataFrame(
+        orthant_signs,
+        columns=model.reversible_rxn_ids,
+    )
+    orthant_weights = pd.DataFrame(result.direction_counts, columns=["weight"])
+
+    result = FreeEnergiesSamplingResult(
+        pd.DataFrame(samples, columns=model.T.reaction_ids),
+        model.compute_psrf(result),
+        orthant_weights.join(orthants),
+        pd.DataFrame(basis_samples, columns=basis_var_names),
+        result.chains,
+    )
+
+    return result
+
+
+def _fill_settings(
+    settings,
+    num_samples,
+    steps,
+    num_chains,
+    num_warmup_steps,
+    max_threads,
+    num_direction_samples,
+):
+    fill_common_sampling_settings(
+        settings, num_samples, steps, num_chains, num_warmup_steps, max_threads
+    )
     settings.steps_thinning_directions = math.floor(
         (settings.num_steps - settings.num_skipped_steps)
         / math.ceil(num_direction_samples / settings.num_chains)
@@ -413,76 +582,34 @@ def sample_drg(
         "Please select a larger number of steps or chains."
     )
 
-    # Generate initial points if needed.
-    if initial_points is None:
-        initial_points = get_initial_points(model, settings.num_chains)
-    assert (
-        initial_points.shape[1] == settings.num_chains
-    ), "Sampling requires the same number of initial points and chains."
 
-    logger.info(
-        f"Starting thermodynamic space sampler with {settings.num_chains} chains and "
-        f"{settings.num_steps} steps."
+def _make_drg_polytope(model: TFSModel) -> Tuple[np.ndarray, np.ndarray]:
+    # Find irreversible reactions.
+    reaction_idxs_T = list(range(len(model.T.reaction_ids)))
+    reaction_idxs_F = [model.F.reaction_ids.index(id) for id in model.T.reaction_ids]
+    only_forward_ids_T = [
+        i for i in reaction_idxs_T if model.F.lb[reaction_idxs_F[i]] >= 0
+    ]
+    only_backward_ids_T = [
+        i for i in reaction_idxs_T if model.F.ub[reaction_idxs_F[i]] <= 0
+    ]
+
+    # Construct direction constraints on the free energies polytope, in the form
+    # G*drg <= h.
+    drg_G = np.vstack(
+        (
+            model.B.to_drg_transform[0][only_forward_ids_T, :],
+            -model.B.to_drg_transform[0][only_backward_ids_T, :],
+        )
+    )
+    drg_h = -np.vstack(
+        (
+            model.B.to_drg_transform[1][only_forward_ids_T, :],
+            -model.B.to_drg_transform[1][only_backward_ids_T, :],
+        )
     )
 
-    try:
-        # TODO: calling the sampler this way is only correct when the thermodynamic
-        # space basis only represents reaction energies.
-        assert model.B.to_drg_transform is not None
-        tfs_result: pb.TFSResult = pb.sample_free_energies(
-            model.B.to_drg_transform[0],
-            model.B.to_drg_transform[1],
-            model.F.S,
-            model.F.lb,
-            model.F.ub,
-            np.array(
-                [[model.F.reaction_ids.index(id) for id in model.T.reaction_ids]],
-                dtype=np.uint32,
-            ).T,
-            initial_points,
-            settings,
-            np.identity(model.B.to_drg_transform[1].size),
-            np.zeros_like(model.B.to_drg_transform[1]),
-            np.identity(model.dimensionality),
-        )
-    except Exception as e:
-        logger.error(f"Sampling of the thermodynamic space failed. {e}")
-        raise SamplingException("Sampling of the thermodynamic space failed.") from e
-    else:
-        logger.info("Sampling completed.")
-        samples = sample_from_chains(tfs_result.chains, num_samples)
-
-        logger.info("Computing PSRFs.")
-        psrf_var_names = model.T.reaction_ids
-        psrf = split_R(tfs_result.chains)
-
-        # Convert the orthants from binary to integer description.
-        orthant_signs = np.unpackbits(tfs_result.directions, bitorder="little", axis=1)[
-            :, : len(model.reversible_rxn_ids)
-        ].astype(np.int8)
-        orthant_signs[orthant_signs == 0] = -1
-
-        orthants = pd.DataFrame(
-            orthant_signs,
-            columns=model.reversible_rxn_ids,
-        )
-        orthant_weights = pd.DataFrame(tfs_result.direction_counts, columns=["weight"])
-
-        result = FreeEnergiesSamplingResult(
-            pd.DataFrame(samples, columns=model.T.reaction_ids),
-            pd.Series(psrf, index=psrf_var_names),
-            orthant_weights.join(orthants),
-        )
-
-        if result.check_convergence(max_psrf):
-            logger.info("Convergence criteria satisfied.")
-        else:
-            logger.warning(
-                "Sampling of the thermodynamic space did not converge. Please try "
-                "increasing the number of steps."
-            )
-
-        return result
+    return drg_G, drg_h
 
 
 def sample_log_conc_from_drg(
@@ -571,19 +698,17 @@ def sample_drg0_from_drg(
 
 
 def _init_us_worker(args_filename):
-    global _us_model
+    global _us_model  # pylint: disable=global-statement
     _us_model = pickle.load(open(args_filename, "rb"))
 
 
-def _sample_orthant(args: Tuple[np.array, np.array, int]) -> pd.DataFrame:
-    global _us_model
+def _sample_orthant(args: Tuple[np.ndarray, np.ndarray, int]) -> pd.DataFrame:
+    global _us_model  # pylint: disable=global-statement
     model = _us_model
-    for i in range(len(model.reactions)):
+    for i, _ in enumerate(model.reactions):
         model.reactions[i].lower_bound = args[0][i]
         model.reactions[i].upper_bound = args[1][i]
-    return sample_flux_space_uniform(
-        model, args[2], num_chains=1, check_convergence=False
-    ).samples
+    return sample_flux_space_uniform(model, args[2], num_chains=1).samples
 
 
 def _get_sample_weight(orthant_weights, orthant_signs, sample_signs):
@@ -694,13 +819,13 @@ def sample_fluxes_from_drg(
 
 
 def _sample_conditional_mvn(
-    mean: np.array,
-    cov_sqrt: np.array,
-    Y_samples: np.array,
+    mean: np.ndarray,
+    cov_sqrt: np.ndarray,
+    Y_samples: np.ndarray,
     X_ids: List[int],
     min_eigenvalue: float = default_min_eigenvalue_tds_basis,
-) -> np.array:
-    """ Samples from the probability density of X conditioned on Y. """
+) -> np.ndarray:
+    """Samples from the probability density of X conditioned on Y."""
     Y_ids = sorted(list(set(range(mean.size)) - set(X_ids)))
 
     # https://en.wikipedia.org/wiki/Schur_complement
@@ -711,7 +836,6 @@ def _sample_conditional_mvn(
     A = a @ a.T
     B = a @ b.T
     C = b @ b.T
-    A_inv = np.linalg.pinv(A, hermitian=True)
     C_inv = np.linalg.pinv(C, hermitian=True)
 
     E_X = mean[X_ids]
