@@ -1,34 +1,35 @@
-import datetime
+"""Uniform sampling of the flux space of a metabolic network."""
+
 import logging
 from typing import List, Tuple, Union
 
-import numpy as np
-import pandas as pd
-
 import _pta_python_binaries as pb
 import cobra
+import numpy as np
+import pandas as pd
 from PolyRound.api import PolyRoundApi, Polytope, StoichiometryParser
 
 from ..constants import (
     default_flux_bound,
     default_max_psrf,
+    default_max_threads,
     default_num_samples,
-    us_steps_multiplier,
 )
 from ..utils import apply_transform
 from .commons import (
-    SamplingException,
+    SamplerInterface,
     SamplingResult,
     apply_to_chains,
     fill_common_sampling_settings,
     sample_from_chains,
     split_R,
 )
+from .convergence_manager import ConvergenceManager
 
 logger = logging.getLogger(__name__)
 
 
-class UniformSamplingModel:
+class UniformSamplingModel(SamplerInterface):
     """Object holding the information necessary to run uniform sampling.
 
     Parameters
@@ -97,7 +98,7 @@ class UniformSamplingModel:
     def from_cobrapy_model(
         cls, model: cobra.Model, infinity_flux_bound: float = default_flux_bound
     ) -> "UniformSamplingModel":
-        """Builds a uniform sampling model from a cobrapy model.
+        """Builds a uniform sampler model from a cobrapy model.
 
         Parameters
         ----------
@@ -114,49 +115,115 @@ class UniformSamplingModel:
         polytope = StoichiometryParser.extract_polytope(model, infinity_flux_bound)
         return cls(polytope, [r.id for r in model.reactions])
 
+    def get_initial_points(self, num_points: int) -> np.array:
+        """Gets initial points for sampling fluxes.
 
-def get_initial_points(model: "UniformSamplingModel", num_points: int) -> np.array:
-    """Gets initial points for sampling fluxes.
+        Parameters
+        ----------
+        model : UniformSamplingModel
+            The model to sample.
+        num_points : int
+            Number of initial points to generate.
 
-    Parameters
-    ----------
-    model : UniformSamplingModel
-        The model to sample.
-    num_points : int
-        Number of initial points to generate.
+        Returns
+        -------
+        np.array
+            Array containing the initial points.
+        """
+        # Find the radius of the hypersphere inscribed in the polytope.
+        distances = self.h / np.linalg.norm(self.G, axis=1)
+        radius = np.min(distances)
 
-    Returns
-    -------
-    np.array
-        Array containing the initial points.
-    """
-    # Find the radius of the hypersphere inscribed in the polytope.
-    distances = model.h / np.linalg.norm(model.G, axis=1)
-    radius = np.min(distances)
+        # Sample random directions and scale them to a random length inside the hypersphere.
+        samples = np.random.rand(self.dimensionality, num_points)
+        length = np.random.rand(1, num_points) ** (
+            1 / self.dimensionality
+        ) / np.linalg.norm(samples, axis=0)
+        samples = samples * np.diag(length) * radius
 
-    # Sample random directions and scale them to a random length inside the hypersphere.
-    samples = np.random.rand(model.dimensionality, num_points)
-    length = np.random.rand(1, num_points) ** (
-        1 / model.dimensionality
-    ) / np.linalg.norm(samples, axis=0)
-    samples = samples * np.diag(length) * radius
+        logger.info(f"Generated {num_points} initial points.")
+        return samples
 
-    logger.info(f"Generated {num_points} initial points.")
-    return samples
+    def simulate(
+        self,
+        settings: pb.SamplerSettings,
+        initial_points: np.ndarray,
+        directions_transform: np.ndarray,
+    ) -> np.ndarray:
+        """Run the sampler with the given parameters.
+
+        Parameters
+        ----------
+        settings : pb.SamplerSettings
+            Sampling settings.
+        initial_points : np.ndarray
+            The initial points for the chains.
+        directions_transform : np.ndarray
+            The transform for the directions sampler.
+        """
+
+        chains = pb.sample_flux_space_uniform(
+            self.G,
+            self.h,
+            directions_transform,
+            initial_points,
+            settings,
+        )
+        return chains
+
+    def compute_psrf(self, result: np.ndarray) -> pd.Series:
+        """Compute the potential scale reduction factors for the variables of interest
+        on a given set of chains.
+
+        Parameters
+        ----------
+        result : np.ndarray
+            The result of the sampling function.
+
+        Returns
+        -------
+        pd.Series
+            The computed potential scale reduction factors.
+        """
+        logger.debug("Computing PSRFs.")
+        psrf_var_names = [
+            "var" + str(i) for i in range(self.dimensionality)
+        ] + self.reaction_ids
+        psrf = np.hstack(
+            [split_R(result), split_R(apply_to_chains(result, self.to_fluxes))]
+        )
+        return pd.Series(psrf, index=psrf_var_names)
+
+    def get_chains(self, result: np.ndarray) -> np.ndarray:
+        """Extract the simulated chains from a given result.
+
+        Parameters
+        ----------
+        result : np.ndarray
+            The result of the native sampling function.
+
+        Returns
+        -------
+        np.ndarray
+            The simulated chains.
+        """
+
+        return result
 
 
 def sample_flux_space_uniform(
     model: Union[cobra.Model, "UniformSamplingModel"],
     num_samples: int = default_num_samples,
-    initial_points: np.array = None,
-    num_steps: int = -1,
-    num_warmup_steps: int = -1,
-    num_chains: int = -1,
-    log_interval: datetime.timedelta = None,
+    max_steps: int = -1,
     max_psrf: float = default_max_psrf,
-    check_convergence: bool = True,
+    num_chains: int = -1,
+    initial_points: np.array = None,
+    num_initial_steps: int = -1,
+    max_threads: int = default_max_threads,
+    convergence_manager: ConvergenceManager = None,
 ) -> SamplingResult:
-    """Sample steady state fluxes in the given model.
+    """Sample steady state fluxes in the given model. The sampler run until either
+    max_steps or max_psrf is reached.
 
     Parameters
     ----------
@@ -164,20 +231,20 @@ def sample_flux_space_uniform(
         The model to sample.
     num_samples : int, optional
         Number of samples to draw.
-    initial_points : np.array, optional
-        The initial points for the chains.
-    num_steps : int, optional
-        The number of steps to use for each chain.
-    num_warmup_steps : int, optional
-        The number of burn-in steps to discard.
-    num_chains : int, optional
-        The number of chains to simulate.
-    log_interval : datetime.timedelta, optional
-        How often should the sampler log the progress.
+    max_steps : int, optional
+        The maximum number fo steps to simulate.
     max_psrf : float, optional
         Maximum value of the PSRFs for convergence.
-    check_convergence : bool, optional
-        Whether to check for convergence or not.
+    num_chains : int, optional
+        The number of chains to simulate.
+    initial_points : np.array, optional
+        The initial points for the chains.
+    num_initial_steps: int, optional
+        Initial chains length.
+    max_threads : int, optional
+        The maximum number of parallel threads to use.
+    convergence_manager : ConvergenceManager, optional
+        The object to use to monitor and improve convergence.
 
     Returns
     -------
@@ -189,71 +256,58 @@ def sample_flux_space_uniform(
     SamplingException
         If sampling fails.
     """
+    # Validate and process input settings.
     if isinstance(model, cobra.Model):
         model = UniformSamplingModel.from_cobrapy_model(model)
-
-    if num_steps < 0:
-        num_steps = model.dimensionality ** 2 * us_steps_multiplier
+    assert num_samples > 0
+    max_steps = max_steps if max_steps >= 1 else 2 ** 63
+    assert max_psrf > 1.0
     if initial_points is not None and num_chains < 0:
         num_chains = initial_points.shape[1]
+    num_initial_steps = (
+        num_initial_steps
+        if num_initial_steps > 0
+        else model.dimensionality ** 2 + num_samples
+    )
+    assert max_threads > 0
+    convergence_manager = ConvergenceManager(model, num_initial_steps, False)
 
-    # Fill the settings.
+    # Construct auxiliary functions.
     settings = pb.UniformSamplerSettings()
-    fill_common_sampling_settings(
-        settings,
-        "",
-        num_samples,
-        num_steps,
-        num_chains,
-        num_warmup_steps,
-        log_interval,
+    update_settings_function = lambda s, steps: fill_common_sampling_settings(
+        s, num_samples, steps, num_chains, 2, max_threads
+    )
+    update_settings_function(settings, num_initial_steps)
+    make_sampling_result_function = lambda r: _make_sampling_result(
+        model, r, num_samples
     )
 
     # Generate the initial points if needed.
-    initial_points = initial_points or get_initial_points(model, settings.num_chains)
-    assert (
-        initial_points.shape[1] == settings.num_chains
-    ), "Sampling requires the same number of initial points and chains."
+    if initial_points is None:
+        initial_points = model.get_initial_points(settings.num_chains)
 
-    logger.info(
-        f"Starting uniform sampler with {settings.num_chains} chains and "
-        f"{settings.num_steps} steps."
+    # Run the sampler.
+    result = convergence_manager.run(
+        settings,
+        update_settings_function,
+        make_sampling_result_function,
+        initial_points,
+        max_steps,
+        max_psrf,
     )
+    return result
 
-    try:
-        chains = pb.sample_flux_space_uniform(
-            model.G,
-            model.h,
-            np.identity(model.dimensionality),
-            initial_points,
-            settings,
-        )
-    except Exception as e:
-        logger.error(f"Uniform sampling failed. {e}")
-        raise SamplingException("Uniform sampling failed.") from e
-    else:
-        logger.info("Sampling completed.")
-        samples = model.to_fluxes(sample_from_chains(chains, num_samples).T).T
 
-        logger.info("Computing PSRFs.")
-        psrf_var_names = [
-            "var" + str(i) for i in range(model.dimensionality)
-        ] + model.reaction_ids
-        psrf = np.hstack(
-            [split_R(chains), split_R(apply_to_chains(chains, model.to_fluxes))]
-        )
+def _make_sampling_result(
+    model: "UniformSamplingModel", result: np.ndarray, num_samples: int
+) -> SamplingResult:
+    basis_var_names = ["var" + str(i) for i in range(model.dimensionality)]
+    basis_samples = sample_from_chains(result, num_samples)
+    samples = model.to_fluxes(basis_samples.T).T
 
-        result = SamplingResult(
-            pd.DataFrame(samples, columns=model.reaction_ids),
-            pd.Series(psrf, index=psrf_var_names),
-        )
-        if check_convergence:
-            if result.check_convergence(max_psrf):
-                logger.info("Convergence criteria satisfied.")
-            else:
-                logger.warning(
-                    "Uniform sampling did not converge. Please try increasing the number "
-                    "of steps."
-                )
-
-        return result
+    return SamplingResult(
+        pd.DataFrame(samples, columns=model.reaction_ids),
+        model.compute_psrf(result),
+        pd.DataFrame(basis_samples, columns=basis_var_names),
+        result,
+    )
