@@ -1,24 +1,28 @@
+"General utility functions."
+
 import logging
 import math
 from pathlib import Path
 from typing import List, Optional, Tuple, Union
 
+import cobra
 import efmtool
 import numpy as np
 import numpy.linalg as la
-from equilibrator_cache import CompoundCache, create_compound_cache_from_zenodo
-
-import cobra
+import pkg_resources
 from cobra.flux_analysis.variability import (
     find_blocked_reactions,
     flux_variability_analysis,
 )
 from cobra.util import create_stoichiometric_matrix
+from equilibrator_cache import CompoundCache, create_compound_cache_from_zenodo
 
-from .constants import Q, default_flux_bound, default_min_eigenvalue_tds_basis
+from .constants import Q, default_min_eigenvalue_tds_basis
 from .flux_space import FluxSpace
 from .metabolite_interpreter import MetaboliteInterpreter
 
+logger = logging.getLogger(__name__)
+
 
 def enable_all_logging(level: int = logging.INFO):
     """Enable detailed logging in PTA and its dependencines. This is useful to debug
@@ -30,7 +34,7 @@ def enable_all_logging(level: int = logging.INFO):
         The desired logging level, by default logging.INFO
     """
     logging.basicConfig()
-    logging.getLogger().setLevel(logging.INFO)
+    logging.getLogger().setLevel(level)
 
 
 def floor(value: float, decimals: int = 0) -> float:
@@ -87,7 +91,10 @@ def get_internal_reactions(model: cobra.Model) -> List[str]:
 
 
 def tighten_model_bounds(
-    model: cobra.Model, margin: float = 1e-4, round_to_digits: int = 6
+    model: cobra.Model,
+    margin: float = 1e-4,
+    round_to_digits: int = 6,
+    prevent_loops: bool = False,
 ):
     """Apply FVA to a model to reduce the range of the flux bounds. The FVA result (plus
     a margin) is applied to each reaction if it is tighter than the bounds in the model.
@@ -104,12 +111,23 @@ def tighten_model_bounds(
         Number of digits to which tighter bounds should be rounded to. This limits the
         range of the coefficients for optimization problems. By default 6.
     """
-    fva_result = flux_variability_analysis(model, fraction_of_optimum=0.0)
+    fva_result = flux_variability_analysis(
+        model, loopless=prevent_loops, fraction_of_optimum=0.0
+    )
+
     for r in model.reactions:
         lb = fva_result.loc[r.id, "minimum"]
         ub = fva_result.loc[r.id, "maximum"]
-        r.lower_bound = max(r.lower_bound, floor(lb - margin, round_to_digits))
-        r.upper_bound = min(r.upper_bound, ceil(ub + margin, round_to_digits))
+
+        if lb >= 0 and lb <= margin:
+            r.lower_bound = max(r.lower_bound, 0.0)
+        else:
+            r.lower_bound = max(r.lower_bound, floor(lb - margin, round_to_digits))
+
+        if ub <= 0 and ub >= -margin:
+            r.upper_bound = min(r.upper_bound, 0.0)
+        else:
+            r.upper_bound = min(r.upper_bound, ceil(ub + margin, round_to_digits))
 
 
 def get_candidate_thermodynamic_constraints(
@@ -151,20 +169,17 @@ def get_candidate_thermodynamic_constraints(
     metabolite_interpreter = metabolite_interpreter or MetaboliteInterpreter()
     exclude_compartments = exclude_compartments or list()
     ccache = ccache or create_compound_cache_from_zenodo()
+    unknown_compounds_found = False
 
-    for i in range(len(flux_space.reaction_ids)):
+    for i, rxn_id in enumerate(flux_space.reaction_ids):
         # Ignore reactions that have no product or no substrate.
         is_candidate = not np.all(flux_space.S[:, [i]] >= 0) and not np.all(
             flux_space.S[:, [i]] <= 0
         )
 
         # Ignore potential biomass reactions.
-        is_candidate = (
-            is_candidate and not "biomass" in flux_space.reaction_ids[i].lower()
-        )
-        is_candidate = (
-            is_candidate and not "growth" in flux_space.reaction_ids[i].lower()
-        )
+        is_candidate = is_candidate and not "biomass" in rxn_id.lower()
+        is_candidate = is_candidate and not "growth" in rxn_id.lower()
 
         # Ignore reactions with metabolites in the excluded compartments.
         is_candidate = is_candidate and all(
@@ -181,12 +196,23 @@ def get_candidate_thermodynamic_constraints(
             metabolite_interpreter.read(m).key for m in rxn_metabolite_ids
         ]
         rxn_compounds = [ccache.get_compound(m) for m in rxn_metabolite_keys]
-        is_h2o = [ccache.is_water(c) for c in rxn_compounds]
-        is_candidate = is_candidate and not all(is_h2o)
+        unknown_compounds_found = unknown_compounds_found or any(
+            [c is None for c in rxn_compounds]
+        )
+        is_h2o_or_proton = [
+            c is not None and (ccache.is_water(c) or ccache.is_proton(c))
+            for c in rxn_compounds
+        ]
+        is_candidate = is_candidate and not all(is_h2o_or_proton)
 
         if is_candidate:
             candidate_ids.append(flux_space.reaction_ids[i])
 
+    if unknown_compounds_found:
+        logger.warning(
+            "One or more metabolites were not found in the compound cache. Automated "
+            "selection of thermodynamically constrained reactions may be unreliable."
+        )
     return candidate_ids
 
 
@@ -253,7 +279,7 @@ def get_reactions_in_internal_cycles(
 
 def get_internal_cycles(
     model: cobra.Model, biomass_name: Optional[str] = None, atpm_name: str = "ATPM"
-) -> np.array:
+) -> np.ndarray:
     """Uses :code:`efmtool` to enumerate all the internal cycles in the network.
 
     model : cobra.Model
@@ -265,7 +291,7 @@ def get_internal_cycles(
 
     Returns
     -------
-    np.array
+    np.ndarray
         A n-by-e matrix, where e in the number of EFMs and n is the number of reactions,
         where each column is an EFM representing an internal cycle.
     """
@@ -273,29 +299,26 @@ def get_internal_cycles(
         results = model.reactions.query(
             lambda r: "biomass" in r.id.lower() or "growth" in r.id.lower()
         )
-        assert len(results) == 1, (
+        assert len(results) <= 1, (
             f"Found {len(results)} candidate biomass "
-            "reactions, but need exactly one. Please specify the name of the "
+            "reactions, but need at most one. Please specify the name of the "
             "biomass reaction."
         )
-        biomass_name = results[0].id
-    assert model.reactions.has_id(biomass_name), (
-        f"Reaction {biomass_name} " "not found in the model"
-    )
-    assert model.reactions.has_id(atpm_name), (
-        f"Reaction {atpm_name} " "not found in the model"
-    )
+        if len(results) == 1:
+            biomass_name = results[0].id
 
     internal_model = model.copy()
     for reaction in internal_model.boundary:
         reaction.lower_bound = 0
         reaction.upper_bound = 0
-    atpm = internal_model.reactions.get_by_id(atpm_name)
-    atpm.lower_bound = 0
-    atpm.upper_bound = 0
-    biomass = internal_model.reactions.get_by_id(biomass_name)
-    biomass.lower_bound = 0
-    biomass.upper_bound = 0
+    if model.reactions.has_id(atpm_name):
+        atpm = internal_model.reactions.get_by_id(atpm_name)
+        atpm.lower_bound = 0
+        atpm.upper_bound = 0
+    if model.reactions.has_id(biomass_name):
+        biomass = internal_model.reactions.get_by_id(biomass_name)
+        biomass.lower_bound = 0
+        biomass.upper_bound = 0
 
     # Find reactions that can have flux in the model even if the exchanges are
     # set to zero.
@@ -311,6 +334,10 @@ def get_internal_cycles(
         )
         raise
 
+    # If the internal model contains no reaction then there is no cycle and we are done.
+    if len(internal_model.reactions) == 0:
+        return np.empty((0, 0))
+
     # First tighten the model bounds to discover all the true reversibilities. Since
     # efmtool assumes irreversibility only in the forward direction we must reverse th
     # reactions that only operate in the backward direction.
@@ -333,9 +360,8 @@ def get_internal_cycles(
     # Finally, we map the EFMs to the original model.
     efms[to_reverse, :] = -efms[to_reverse, :]
     cycles = np.zeros((len(model.reactions), efms.shape[1]))
-    for i in range(len(internal_model.reactions)):
-        reaction_id = internal_model.reactions[i].id
-        cycles[model.reactions.index(reaction_id), :] = efms[i, :]
+    for i, r in enumerate(internal_model.reactions):
+        cycles[model.reactions.index(r.id), :] = efms[i, :]
 
     return cycles
 
@@ -452,20 +478,22 @@ def qrvector(elements: List[Q]) -> Q:
         return values * units
 
 
-def apply_transform(value: np.array, transform: Tuple[np.array, np.array]) -> np.array:
+def apply_transform(
+    value: np.ndarray, transform: Tuple[np.ndarray, np.ndarray]
+) -> np.ndarray:
     """Applies an affine transform to a matrix.
 
     Parameters
     ----------
-    value : np.array
+    value : np.ndarray
         The matrix to be transformed.
-    transform : Tuple[np.array, np.array]
+    transform : Tuple[np.ndarray, np.ndarray]
         Tuple describing the linear (transformation matrix) and affine (translation
         vector) of the transform.
 
     Returns
     -------
-    np.array
+    np.ndarray
         The transformed matrix.
     """
     return transform[0] @ value + transform[1]
@@ -498,20 +526,20 @@ def get_path(path: Union[Path, str]) -> Path:
 
 
 def covariance_square_root(
-    covariance: np.array, min_eigenvalue: float = default_min_eigenvalue_tds_basis
-) -> np.array:
+    covariance: np.ndarray, min_eigenvalue: float = default_min_eigenvalue_tds_basis
+) -> np.ndarray:
     """Gets a full-rank square root of a covariance matrix.
 
     Parameters
     ----------
-    covariance : np.array
+    covariance : np.ndarray
         The input covariance matrix.
     min_eigenvalue : float, optional
         Minimum eigenvalue to keep during the truncated EVD.
 
     Returns
     -------
-    np.array
+    np.ndarray
         A full-rank square root of the covariance.
     """
 
@@ -519,8 +547,7 @@ def covariance_square_root(
     assert np.all(np.abs(np.imag(w)) < min_eigenvalue) and np.all(
         np.abs(np.imag(v * np.real(w))) < min_eigenvalue
     ), (
-        "The "
-        "eigenvalue decomposition of the covariance matrix has "
+        "The eigenvalue decomposition of the covariance matrix has "
         "imaginary parts larger than the specified minimum eigenvalue."
     )
     assert np.min(w) > -min_eigenvalue, (
@@ -535,3 +562,65 @@ def covariance_square_root(
     v = v[:, selected_dims]
 
     return v @ np.diag(np.sqrt(w))
+
+
+def load_example_model(model_name: str) -> cobra.Model:
+    """Load an example SBML model in cobrapy.
+
+    Parameters
+    ----------
+    model_name : str
+        Name of the model file (without extension). This can be any of the models in
+        pta/data/models.
+
+    Returns
+    -------
+    cobra.Model
+        The loaded model in cobrapy format.
+    """
+    file_name = f"data/models/{model_name}.xml"
+    if not pkg_resources.resource_exists("pta", file_name):
+        logger.warning(f"Model {model_name} does not exist.")
+        return None
+    else:
+        return cobra.io.read_sbml_model(
+            pkg_resources.resource_filename("pta", file_name)
+        )
+
+
+def find_rotation_matrix(x: np.ndarray, y: np.ndarray) -> np.ndarray:
+    """Compute a matrix that rotates the vector x onto vector y (without scaling).
+
+    Parameters
+    ----------
+    x : np.ndarray
+        The starting vector.
+    y : np.ndarray
+        The destination vector.
+
+    Returns
+    -------
+    np.ndarray
+        The computed rotation matrix.
+    """
+    # pylint: disable=line-too-long
+    # Adapted from:
+    # https://math.stackexchange.com/questions/598750/finding-the-rotation-matrix-in-n-dimensions
+    x = x.flatten()
+    y = y.flatten()
+
+    u = x / np.linalg.norm(x)
+    v = y - np.dot(u, y) * u
+    v = v / np.linalg.norm(v)
+
+    cos_t = np.dot(x, y) / np.linalg.norm(x) / np.linalg.norm(y)
+    sin_t = np.sqrt(1 - cos_t ** 2)
+
+    return (
+        np.identity(len(x))
+        - np.outer(u, u)
+        - np.outer(v, v)
+        + np.vstack((u, v)).T
+        @ np.array([[cos_t, -sin_t], [sin_t, cos_t]])
+        @ np.vstack((u, v))
+    )
